# URBANITE Traffic Simulation Component 

This component creates and runs the traffic simulations.

## Database service connection

Traffic simulation component connects to the database of the Storage component. 
Relevant environment variables:
- `database_url`
  URL to storage component's database, including port and database name
- `databse_username`
  Storage component's database username
- `database_password`
  Storage component's database password

## Common data folder 

URBANITE Components TS, DSS, and Storage share a common data folder.
The path to the data folder's initial contents is defined in environment variable `storage.data_dir`.

### `storage.data_dir` structure

Before running, populate the directory with data, available here:
[https://portal.ijs.si/nextcloud/s/m8zkQGAwGipS329](Storage component data)

This server has access to the URBANITE Storage component's shared data folder. 
The structure is as follows:
- `data`
  - `<city_id>`
    - `before_simulation`
      - population generation input files
      - travel demand input files
      - preprocessing files (for calibration)
    - `original_input` - predefined scenarios and all their inputs
      - `networks`
        - `<net_id>` - predefined and user generated network go here
          - `network.xml`
          - `plans` - predefined and user generated plans go here since they are dependent on the network
            - `<plan_id>`
              - `plans.xml`
      - `vehicles.xml` - predefined
      - `config.xml` - original config that is copied to simulation folders
    - `simulations`
      - `<simulation_id>` - created when user creates simulation
        - `network.xml`
        - `vehicles.xml`
        - `plans.xml`
        - `config.xml`
        - `results`
          - `<datetime>` - create when user runs the simulation

## API

- GET `/city`

  Returns list of all cities.

- GET `/city/selected`

  Returns the city deployed inReturns the city deployed in

- GET `/city/<city_id>`

  `city_id` is lowercase name of the city. 
  Returns the city entity data.

- GET `/city/<city_id>/scenarios`

  Returns list of scenarios for city. **deprecated**

- GET `/city/<city_id>/coordinates`

  Returns the city centre coordinates.
- GET `/city/<city_id>/networks`

  Returns list of networks for city. **deprecated**

---
- GET `/network`
  
  Returns list of all networks

- GET `/network/<net_id>`

  Returns network entity object.

- POST `/network`
  Creates a new network.
  _payload_: NetworkDTO
  Returns: NetworkDTO, including assigned ID 

- DELETE `/network/<net_id>`

  Deletes network.

- GET `/network/geojson/<net_id>`

  Returns geojson representation of the network for visualization. **WIP**

---
- GET `/scenario`

  Returns list of all scenarios.

- GET `/scenario/<scenario_id>`

  Returns a scenario entity object.

- GET `/scenario/<scenario_id>/simulations`
  
  Returns list of simulations in scenario.  **DEPRECATED**

- POST `/scenario`

  Create a scenario.
  _Payload_: Scenario entity object

---

- GET `/simulation`

  Returns all simulation entity objects

- GET `/simulation/<sim_id>/run`

  Start the pipeline by simulation id.
  1. preprocess population data
  2. run population generation
  3. run travel demand generation **ERROR**
  4. run simulations

- GET `/simulation/<sim_id>`

  Return simulation entity object by id

- GET `/simulation/<sim_id>/<file_name>`

  Returns the file by name for simulation by id **DEPRECATED** 

- POST `/simulation`
  
  Create new simulation
  _Payload_: SimulationDTO object JSON
  Returns: SimulationDTO with assigned ID

- GET `/simulation/<sim_id>/data/<file_name>`
  
  Returns the file by name for simulation by id **DEPRECATED**

- GET `/simulation/<sim_id>/data`

  Returns the path of simulation's data folder **DEPRECATED**


## User actions
- page create simulation - **WIP**
  1. create/choose network on UUI
  2. create/choose plans for selected network on UUI
  3. run simulation
- page results - **WIP**
  1. list simulations
  2. select simulation and visualize results


## Scripts and input/output files for before running simulation
- Bilbao:
  - `bilbao_preprocess.py`
    - Input:
      - ES_2013h_EUSILC.csv, ES_2013p_EUSILC.csv, ES_2013d_EUSILC.csv, ES_2013r_EUSILC.csv
      - numero_habitantes_distrito_barrio_edad_2020.csv
      - ODM_bilbao1.csv
    - Output:
      - ind.csv
      - cons.csv
      - ODM_bilbao_districtLevel.csv
      - ODM_bilbao_districtLevel_reduced.csv

  - `bilbao_population.R`
    - Input:
      - ind.csv
      - cons.csv
    - Output:
      - ind_full.csv
	
  - `TravelDemandGenerationBilbao.java`
    - Input:
      - ODM_bilbao_districtLevel_codes.csv
      - config.xml -> plans.xml, counts.xml, vehicle.xml, bilbao_multimodalnetwork.xml, schedule.xml 
      - ind_full.csv
    - Output:
      - plans.xml

__________________________________________________
- Amsterdam:
  - `amsterdam_preprocess.py`
    - Input:
      - NL_2013h_EUSILC.csv, NL_2013p_EUSILC.csv, NL_2013d_EUSILC.csv, NL_2013r_EUSILC.csv
      - 2021-22-gebieden-1-02.csv
    - Output:
      - #commented microdata_NL.csv
      - ind.csv
      - cons.csv
  - `amsterdam_population.R`
    - Input:
      - ind.csv
      - cons.csv
    - Output:
      - ind_full.csv
  - `TravelDemandAmsterdam`:
    - Input:
      - config -> vehicles.xml. plans_cyclists.xml, amsterdam_fine.xml.gz
      - ind_full.csv
    - Output:
      - plans.xml

__________________________________________________
- Helsinki:
  - `helsinki_preprocess.py`
    - Input:
      - FI_2013h_EUSILC.csv, FI_2013p_EUSILC.csv, FI_2013d_EUSILC.csv, FI_2013r_EUSILC.csv
      - Hki_vaesto_taulu9a.csv
    - Output:
      - microdata_NL.csv
      - ind.csv
      - cons.csv
      - ferries.csv
      - ferriesJatkasaari.csv
  - `helsinki_population.R`
    - Input:
      - ind.csv
      - cons.csv
    - Output:
      - indHelsinki.csv	
  - `TravelDemandGenerationHelsinki`
    - Input:
      - indHelsinki.csv
      - config.xml -> helsinki_multimodalnetwork.xml, plans.xml, vehicles.xml, helsinki_schedule.xml
      - ferriesJatkasaari.csv
    - Output:
      - plans.xml
