package si.ijs.urbanite.simserver.city;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.server.ResponseStatusException;
import si.ijs.urbanite.simserver.entities.*;
import si.ijs.urbanite.simserver.network.NetworkRepository;
import si.ijs.urbanite.simserver.simulation.SimulationRepository;
import si.ijs.urbanite.simserver.simulation.SimulationRunService;


@RestController
@CrossOrigin
@RequestMapping("/traffic-simulation/city")
public class CityController {
    private final static Logger logger = LoggerFactory.getLogger(SimulationRunService.class);

    @Value("${urbanite.city}")
    private String currentCity;
    private final CityRepository cityRepository;
    private final NetworkRepository networkRepository;

    private final SimulationRepository simulationRepository;

    public CityController(CityRepository cityRepository,
                          NetworkRepository networkRepository,
                          SimulationRepository simulationRepository) {
        this.cityRepository = cityRepository;
        this.networkRepository = networkRepository;
        this.simulationRepository = simulationRepository;
    }

    /* Returns list of all cities. */
    @GetMapping("")
    public Iterable<City> getCity() {
        return cityRepository.findAll();
    }

    /* Returns the city deployed in */
    @GetMapping("/selected")
    public City getSelectedCity() {
        return cityRepository.findByCityId(currentCity);
    }

    @GetMapping("{cityId}")
    public City getCity(@PathVariable String cityId){
        City city = cityRepository.findByCityId(cityId);
        if(city == null) throw new ResponseStatusException(HttpStatus.NOT_FOUND, "City with given id does not exists");
        return city;
    }

    @GetMapping("{cityId}/networks")
    public Iterable<Network> getCityNetworks(@PathVariable String cityId) {
        City city = cityRepository.findByCityId(cityId);
        if(city == null) throw new ResponseStatusException(HttpStatus.NOT_FOUND, "City with given id does not exists");
        return this.networkRepository.findByCity(city);
    }

    @GetMapping("{cityId}/coordinates")
    public Coordinates getCityCoordinates(@PathVariable String cityId){
        City city = cityRepository.findByCityId(cityId);
        if(city == null) throw new ResponseStatusException(HttpStatus.NOT_FOUND, "City with given id does not exists");
        return city.getCoordinates();
    }

//    @GetMapping("{cityId}/simulations")
//    public Iterable<Simulation> getCitySimulations(@PathVariable String cityId){
//        City city = cityRepository.findByCityId(cityId);
//        if(city == null) throw new ResponseStatusException(HttpStatus.NOT_FOUND, "City with given id does not exists");
//        Iterable<Simulation> simulations = simulationRepository.findByCity(city);
//        return simulations;
//    }

}
