package si.ijs.urbanite.simserver.district_closer;

import org.json.simple.JSONArray;
import org.json.simple.JSONObject;
import org.json.simple.parser.JSONParser;
import org.matsim.api.core.v01.Coord;
import org.matsim.api.core.v01.Id;
import org.matsim.api.core.v01.Scenario;
import org.matsim.api.core.v01.network.Link;
import org.matsim.api.core.v01.network.Network;
import org.matsim.api.core.v01.network.Node;
import org.matsim.core.config.Config;
import org.matsim.core.config.ConfigUtils;
import org.matsim.core.scenario.ScenarioUtils;
import org.matsim.core.utils.geometry.CoordinateTransformation;
import org.matsim.core.utils.geometry.transformations.TransformationFactory;
import org.poly2tri.geometry.polygon.Polygon;
import org.poly2tri.geometry.polygon.PolygonPoint;
import org.poly2tri.triangulation.TriangulationPoint;
import org.matsim.core.network.NetworkUtils;

import java.io.File;
import java.io.FileReader;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.*;


public class DistrictCloser {

    public String city_id;
    public String EPSG;


    public String setCorrespondingEPSG(String city){
        switch (city) {
            case "amsterdam": {
                return "EPSG:32631";
            }
            case "bilbao":
            {
                return "EPSG:32630";
            }
            case "helsinki":{
                return "EPSG:32634";
            }
            case "messina": {
                return "EPSG:32633";
            }
        }
        // If it reaches here, wrong city_id was passed
        return null;
    }

    public void setNewParams(String city_id){
        this.city_id = city_id;
        this.EPSG = setCorrespondingEPSG(city_id);
    }

    public Coord transform_coords(double x, double y){
        CoordinateTransformation transformation = TransformationFactory.getCoordinateTransformation(
                TransformationFactory.WGS84, this.EPSG);
        return transformation.transform(new Coord(x, y));

    }
    public HashMap<String, Polygon> createPolygons(){
        HashMap<String, Polygon> polygons = new HashMap<>();

        // TODO download from data storage gtfsShape
        // read shape file
        JSONParser parser = new JSONParser();
        try {
            // TODO add names for all cities, not just Bilbao

            Path path = Paths.get("/home/simserver/data/bilbao/before_simulation/district_shapes.json");
            Object object = parser.parse(new FileReader(path.toFile()));

            JSONArray districts = (JSONArray) object;

            Iterator<JSONObject> iterator = districts.iterator();
            // going over each district entry
            while (iterator.hasNext()) {
                JSONObject obj = iterator.next();
                // "description" is district name
                String district = (String) obj.get("description");

                ArrayList<PolygonPoint> border = new ArrayList<>();
                // initialised x and y to check for duplicate points
                double x = Double.MAX_VALUE, y = Double.MAX_VALUE;
                PolygonPoint prevPoint = new PolygonPoint(x, y);
                for (Object item : (JSONArray) (((JSONObject) obj.get("location")).get("coordinates"))) {
                    for (Object coordinate_pair : (JSONArray) item) {
                        if (border.size() != 0) {
                            // checking the point is duplicate of the previous point
                            if (x != (double) ((JSONArray) coordinate_pair).get(0)
                                    || y != (double) ((JSONArray) coordinate_pair).get(1)) {
                                x = (double) ((JSONArray) coordinate_pair).get(0);
                                y = (double) ((JSONArray) coordinate_pair).get(1);
                                // checking for duplicates of the starting point
                                if (x == border.get(0).getX() && y == border.get(0).getY()) {
                                    break;
                                }
                            }
                        }
                        x = (double) ((JSONArray) coordinate_pair).get(0);
                        y = (double) ((JSONArray) coordinate_pair).get(1);

                        Coord transformed_point = transform_coords(x,y);
                        PolygonPoint point = new PolygonPoint(transformed_point.getX(), transformed_point.getY());

                        if (prevPoint.getX() != point.getX() || prevPoint.getY() != point.getY()) {
                            border.add(point);
                            prevPoint = point;
                        }
                    }
                    // create the actual polygon object, smallest possible polygon is a
                    // triangle with 3 points
                    if (border.size() > 2) {
                        Polygon polygon = new Polygon(border);
                        polygons.put(district, polygon);
                        // district is a String name, polygon is a triangulatable object
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

        return polygons;
    }
    public List<Id<Link>> cece(Network network, Polygon polygon){

        Map<Id<Link>, ? extends Link> network_links = network.getLinks();
        List<TriangulationPoint> polygon_points = polygon.getPoints();
        PolygonHelperFuncs helperFuncs = new PolygonHelperFuncs();

        List<Id<Link>> links_to_be_removed = new ArrayList<>();

        int counter = 0;
        for (Id<Link> l : network_links.keySet()){
            if (network_links.get(l).getAttributes().size() > 0) {
                Node from_node = network_links.get(l).getFromNode();
                Node to_node = network_links.get(l).getToNode();
                int polygon_point_count =  polygon.pointCount();

                boolean node_one_inside =  helperFuncs.isInside(polygon_points, polygon_point_count, new PolygonPoint(from_node.getCoord().getX(), from_node.getCoord().getY()));
                boolean node_two_inside = helperFuncs.isInside(polygon_points, polygon_point_count, new PolygonPoint(to_node.getCoord().getX(), to_node.getCoord().getY()));

                if(node_one_inside || node_two_inside) {
                    //network.removeLink(l);
                    links_to_be_removed.add(l);
                    counter++;
                }

            }}
        /*
        // Just for visualizing, simulation doesn't need this saved network.
        Path sim_path_for_ea = Paths.get("data",this.city_id,"original_input","networks", "just_for_sim");
        File dir_path = new File(sim_path_for_ea.toString());
        if (!dir_path.exists()) {
            dir_path.mkdirs();
        }
        */
        // NetworkUtils.writeNetwork(network, Paths.get(sim_path_for_ea.toString(), "network_district_closed.xml").toString());
        System.out.println("Links removed: " + counter);
        return links_to_be_removed;
    }

    private class PolygonHelperFuncs {
        public boolean onSegment(PolygonPoint p, PolygonPoint q, PolygonPoint r) {
            if (q.getX() <= Math.max(p.getX(), r.getX()) && q.getX() >= Math.min(p.getX(), r.getX())
                    && q.getY() <= Math.max(p.getY(), r.getY()) && q.getY() >= Math.min(p.getY(), r.getY()))
                return true;
            return false;
        }

        public int orientation(PolygonPoint p, PolygonPoint q, PolygonPoint r) {
            double val = (q.getY() - p.getY()) * (r.getX() - q.getX()) - (q.getX() - p.getX()) * (r.getY() - q.getY());
            if (val == 0)
                return 0;
            return (val > 0) ? 1 : 2;
        }

        public boolean doIntersect(PolygonPoint p1, PolygonPoint q1, PolygonPoint p2, PolygonPoint q2) {
            int o1 = orientation(p1, q1, p2);
            int o2 = orientation(p1, q1, q2);
            int o3 = orientation(p2, q2, p1);
            int o4 = orientation(p2, q2, q1);

            if (o1 != o2 && o3 != o4)
                return true;
            if (o1 == 0 && onSegment(p1, p2, q1))
                return true;
            if (o2 == 0 && onSegment(p1, q2, q1))
                return true;
            if (o3 == 0 && onSegment(p2, p1, q2))
                return true;
            if (o4 == 0 && onSegment(p2, q1, q2))
                return true;
            return false;
        }

        public boolean isInside(List<TriangulationPoint> polygon, int n, PolygonPoint p) {
            if (n < 3)
                return false;
            PolygonPoint extreme = new PolygonPoint(Double.MAX_VALUE, p.getY());
            int count = 0, i = 0;
            do {
                int next = (i + 1) % n;
                if (doIntersect((PolygonPoint) polygon.get(i), (PolygonPoint) polygon.get(next), p, extreme)) {
                    if (orientation((PolygonPoint) polygon.get(i), p, (PolygonPoint) polygon.get(next)) == 0)
                        return onSegment((PolygonPoint) polygon.get(i), p, (PolygonPoint) polygon.get(next));
                    count++;
                }
                i = next;
            } while (i != 0);
            return (count & 1) == 1 ? true : false;
        }
    }
}
