package si.ijs.urbanite.simserver.district_closer;

import org.apache.http.HttpConnection;
import org.matsim.api.core.v01.Id;
import org.matsim.api.core.v01.network.Link;
import org.matsim.core.network.NetworkUtils;
import org.matsim.api.core.v01.network.Network;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.server.ResponseStatusException;
import si.ijs.urbanite.simserver.entities.City;
import si.ijs.urbanite.simserver.entities.Simulation;
import si.ijs.urbanite.simserver.simulation.SimulationController;
import si.ijs.urbanite.simserver.simulation.SimulationRepository;
import si.ijs.urbanite.simserver.simulation.SimulationRunService;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.nio.file.Paths;
import java.util.List;
import java.util.Optional;

@RestController
@CrossOrigin
@RequestMapping("/traffic-simulation/district-closer")
public class DistrictCloserController {
    private final static Logger logger = LoggerFactory.getLogger(SimulationController.class);

    public DistrictCloser districtCloser;
    private final SimulationRunService simulationRunService;
    private final SimulationRepository simulationRepository;

    private static class _EA_run_dto {
        public int area;
        public int start;
        public int duration;
        public String city_id;
        public int sim_id;

    }

    public DistrictCloserController(SimulationRunService simulationRunService, SimulationRepository simulationRepository){
        this.simulationRunService = simulationRunService;
        this.simulationRepository = simulationRepository;
    }

    // The post request has to contain the fields exactly how they are written in the received object class
    /*
{
    "area":1,
    "start":1,
    "duration":1,
    "sim_id":1
}
    */
    @PostMapping("/close_and_run")
    public int run_EA_sim(@RequestBody _EA_run_dto parameters) throws IOException {
        if (this.districtCloser == null){
            this.districtCloser =  new DistrictCloser();
        }
        Optional<Simulation> simulationOptional = this.simulationRepository.findById(parameters.sim_id);
        if (!simulationOptional.isPresent())
            throw new ResponseStatusException(HttpStatus.NOT_FOUND, "Simulation with given id does not exists");
        Simulation simulation = simulationOptional.get();
        City city = simulation.getCity();
        si.ijs.urbanite.simserver.entities.Network network = simulation.getPlan().getNetwork();

        districtCloser.setNewParams(city.getCityId());

        Network net = NetworkUtils.readNetwork(Paths.get("data", city.getCityId(), "original_input", "networks", String.valueOf(network.getId()), "network.xml").toString());

        List<Id<Link>> links_to_be_closed = districtCloser.cece(net, districtCloser.createPolygons().get("Recalde"));

        DistrictCloserParams district_close_params = new DistrictCloserParams(parameters.area, parameters.start, parameters.duration, links_to_be_closed);

//        simulationRunService.runSimulation(simulation, district_close_params);

        // call calc_KPI /dss/kpis/<city_id>/<int:simulation_id>
        String url_request = "dss/kpis/" + city.getCityId() + "/" + simulation.getId();
        URL url = new URL("localhost://" + url_request);
        HttpURLConnection con = (HttpURLConnection) url.openConnection();
        con.setRequestMethod("GET");
        BufferedReader in = new BufferedReader(new InputStreamReader(con.getInputStream()));
        String inputLine;
        StringBuffer content = new StringBuffer();
        while((inputLine = in.readLine()) != null){
            content.append(inputLine);
        }
        in.close();
        logger.error("FINISHED, response is\n\n"+ content);

        return 0;
    }

}
