package si.ijs.urbanite.simserver.entities;

import com.fasterxml.jackson.annotation.JsonIgnore;

import java.util.List;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.SequenceGenerator;

@Entity
public class Network {
    @Id
    @GeneratedValue(strategy = GenerationType.SEQUENCE, generator = "network_id_gen")
    @SequenceGenerator(name = "network_id_gen", sequenceName = "network_id_seq", allocationSize = 1)
    private int id;
    private String name;
    private String description;

    private Double east;
    private Double west;
    private Double north;
    private Double south;

    @ManyToOne()
    @JoinColumn(name = "city", nullable = false)
    private City city;

    @JsonIgnore
    @OneToMany(mappedBy = "network")
    private List<Plan> plans;

    private boolean doneOsmProcessing;
    private boolean doneMATSimProcessing;

    @Column(columnDefinition = "TEXT")
    private String conversionError;

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public Network() {
    }

    public Network(String name, String description, City city, double north, double south, double east, double west) {
        this.name = name;
        this.description = description;
        this.city = city;

        this.east = east;
        this.west = west;
        this.north = north;
        this.south = south;

        this.doneOsmProcessing = false;
        this.doneMATSimProcessing = false;
        this.conversionError = "";
    }

    // For setup
    public Network(int id, String name, String description, City city) {
        this.id = id;
        this.name = name;
        this.description = description;
        this.city = city;

        this.doneOsmProcessing = true;
        this.doneMATSimProcessing = true;
    }

    public int getId() {
        return id;
    }

    public void setId(int id) {
        this.id = id;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public Double getEast() {
        return east;
    }

    public void setEast(Double east) {
        this.east = east;
    }

    public Double getWest() {
        return west;
    }

    public void setWest(Double west) {
        this.west = west;
    }

    public Double getNorth() {
        return north;
    }

    public void setNorth(Double north) {
        this.north = north;
    }

    public Double getSouth() {
        return south;
    }

    public void setSouth(Double south) {
        this.south = south;
    }

    public City getCity() {
        return city;
    }

    public void setCity(City city) {
        this.city = city;
    }

    public List<Plan> getPlans() {
        return plans;
    }

    public void setPlans(List<Plan> plans) {
        this.plans = plans;
    }

    public boolean isDoneOsmProcessing() {
        return doneOsmProcessing;
    }

    public void setDoneOsmProcessing(boolean doneOsmProcessing) {
        this.doneOsmProcessing = doneOsmProcessing;
    }

    public boolean isDoneMATSimProcessing() {
        return doneMATSimProcessing;
    }

    public void setDoneMATSimProcessing(boolean doneMATSimProcessing) {
        this.doneMATSimProcessing = doneMATSimProcessing;
    }

    public String getConversionError() {
        return conversionError;
    }

    public void setConversionError(String conversionError) {
        this.conversionError = conversionError;
    }

    @JsonIgnore
    public String getNetworkFileOSM() {
        return String.format("%d-%s.osm", this.id, this.name.replace(" ", "-")).toLowerCase();
    }

    @JsonIgnore
    public String getNetworkFile() {
        return String.format("%d-%s.xml", this.id, this.name.replace(" ", "-")).toLowerCase();
    }
}
