package si.ijs.urbanite.simserver.entities;

import com.fasterxml.jackson.annotation.JsonIgnore;

import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.SequenceGenerator;

@Entity
public class Simulation {

    public enum Status {
        CREATED,
        RUNNING,
        FINISHED,
        ERROR
    }

    @Id
    @GeneratedValue(strategy = GenerationType.SEQUENCE,
            generator = "simulation_id_gen")
    @SequenceGenerator(name = "simulation_id_gen",
            sequenceName = "simulation_id_seq", allocationSize = 1)
    private int id;
    private String name;
    private String description;
    private Status status;


    @ManyToOne
    @JoinColumn(name = "city", nullable = false)
    private City city;

    @JsonIgnore
    @ManyToOne
    @JoinColumn(name = "plan", nullable = false)
    private Plan plan;

    public Simulation() {
    }

    public Simulation(String name, String description, City city) {
        this.name = name;
        this.description = description;
        this.city = city;
    }

    public Simulation(String name, String description, Plan plan, City city) {
        this.name = name;
        this.description = description;
        this.plan = plan;
        this.city = city;
    }

    public int getId() {
        return id;
    }

    public void setId(int id) {
        this.id = id;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public Status getStatus() {
        return status;
    }

    public void setStatus(Status status) {
        this.status = status;
    }

    public City getCity() {
        return city;
    }

    public void setCity(City city) {
        this.city = city;
    }

    public Plan getPlan() {
        return plan;
    }

    public void setPlan(Plan plan) {
        this.plan = plan;
    }
}
