package si.ijs.urbanite.simserver.scripts;

import org.jfree.data.json.impl.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.web.bind.annotation.*;
import si.ijs.urbanite.simserver.simulation.SimulationController;
import si.ijs.urbanite.simserver.simulation.SimulationWithParamsDTO;

import javax.validation.Valid;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringWriter;


@RestController
@CrossOrigin
@RequestMapping("/traffic-simulation/scripts")
public class ScriptsController {

    private final static Logger logger = LoggerFactory.getLogger(SimulationController.class);

    private final ScriptsService scriptsService;
    public ScriptsController(ScriptsService scriptsService) {
        this.scriptsService = scriptsService;
    }

    @GetMapping("/city/{city_name}/preprocess_data")
    public int handlePreprocessScript(@PathVariable String city_name) throws IOException {
        try {
            return scriptsService.runPreprocessing(city_name);
        }
        catch (Exception e){
            return -1;
        }
    }

    @GetMapping("/city/{city_name}/generate_population")
    public int handleGeneratePopulationScript(@PathVariable String city_name) throws IOException {
        try {
            return scriptsService.runGeneratePopulation(city_name);
        }
        catch (Exception e){
            return -1;
        }
    }

    @PostMapping("/city/{city_name}/travel_demand")
    public JSONObject handleTravelDemandScript(@PathVariable String city_name,
                                        @RequestBody @Valid TravelDemandDTO travelDemandDTO) throws IOException {
        String name = travelDemandDTO.getName();
        String networkId = travelDemandDTO.getNetworkId();
        int simulationId = travelDemandDTO.getSimulationId();
        int errno = 0;
        JSONObject response = new JSONObject();


        //TODO to be fixed, not null
        SimulationWithParamsDTO paramsDTO = null;


        try {
            errno = scriptsService.runTravelDemand(city_name, networkId, String.valueOf(simulationId), name, paramsDTO);
            response.put("status", errno);
            response.put("message", "Travel demand successfully generated.");
            if (errno != 0) {
                response.put("message", "Travel demand generation failed.");
            }
        }
        catch (Exception e){
            StringWriter sw = new StringWriter();
            PrintWriter pw = new PrintWriter(sw);
            e.printStackTrace(pw);
            logger.error(sw.toString());
            response.put("status", -2);
            response.put("message", e.getMessage());
        }

        return response;
    }
}

