package si.ijs.urbanite.simserver.travel_demand;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ArrayNode;
import org.matsim.api.core.v01.Coord;
import org.matsim.core.utils.geometry.CoordinateTransformation;
import org.matsim.core.utils.geometry.transformations.TransformationFactory;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.concurrent.atomic.AtomicLong;

public class Facilities {
    private static final AtomicLong idGenerator = new AtomicLong(0);

    public static long getNextId() {
        return idGenerator.incrementAndGet();
    }

    public static void createFacilityFromPoint(FileWriter fileWriter, String path, String type) throws IOException {


        File file = new File(path);
        CoordinateTransformation transformation = TransformationFactory.getCoordinateTransformation(
                TransformationFactory.WGS84,"EPSG:32630");
        ObjectMapper mapper = new ObjectMapper();
        JsonNode rootNode = mapper.readTree(file);

        // Get the features array from the GeoJSON
        ArrayNode features = (ArrayNode) rootNode.get("features");

        // Iterate over each feature in the array
        for (JsonNode feature : features) {
            // Get the geometry of the feature
            JsonNode geometry = feature.get("geometry");

            // Get the coordinates of the geometry
            JsonNode coordinates = geometry.get("coordinates");
            String typePolygonPoint = geometry.get("type").asText();

            if (typePolygonPoint.matches("Polygon")){
                double x=0.0;
                double y=0.0;
                int counter=0;
                //iterate over coordinate, average them
                if (coordinates.isArray()){
                    for (JsonNode node:coordinates){
                        if (node.isArray()){
                            for (JsonNode node1:node){
                                x+=node1.get(0).asDouble();
                                y+=node1.get(1).asDouble();
                                counter++;
                            }
                        }
                    }
                }
                x=x/counter;
                y=y/counter;

                Coord coord = transformation.transform(new Coord(x,y));
                long id = getNextId();
                String facility = "<facility id=\""+id+"\" x=\""+ coord.getX()+"\" y=\""+coord.getY()+"\">\n";
                String activity2 = "<activity type=\""+type+"\"/>\n";
                String facilityClose = "</facility>\n";
                fileWriter.append(facility+activity2+facilityClose);

            }

            else if (typePolygonPoint.matches("Point")){
                double x =  coordinates.get(0).asDouble();
                double y = coordinates.get(1).asDouble();
                Coord coord = transformation.transform(new Coord(x,y));
                long id = getNextId();
                String facility = "<facility id=\""+id+"\" x=\""+ coord.getX()+"\" y=\""+coord.getY()+"\">\n";
                String activity1 = "<activity type=\"work\"/>\n";
                String activity2 = "<activity type=\""+type+"\"/>\n";
                String facilityClose = "</facility>\n";
                fileWriter.append(facility+activity1+activity2+facilityClose);

            }



        }
    }

    public static void createFacilityFromPolygon() {


    }

    //TODO add home, education, work, other, leisure to onedrive
    public static void createFacility(String city) throws IOException {
        String homePath = Paths.get("data", city, "before_simulation", "home.geojson").toString();
        String educationPath = Paths.get("data", city, "before_simulation", "education.geojson").toString();
        String leisurePath = Paths.get("data", city, "before_simulation", "leisure.geojson").toString();
        String otherPath = Paths.get("data", city, "before_simulation", "other.geojson").toString();

        String facilitiesPath = Paths.get("data", city, "original_input", "facilities.xml").toString();


        FileWriter fw = new FileWriter(facilitiesPath);
        String header = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" +
                "<!DOCTYPE facilities SYSTEM \"http://www.matsim.org/files/dtd/facilities_v2.dtd\">\n" +
                "<facilities>";
        fw.write(header);

        createFacilityFromPoint(fw,educationPath,"education");
        createFacilityFromPoint(fw,leisurePath,"leisure");
        createFacilityFromPoint(fw,otherPath,"other");
        createFacilityFromPoint(fw,homePath,"home");


        fw.append("</facilities>");
        fw.close();



    }

    public static void main(String[] args) throws IOException {
     createFacility("bilbao");

    }
}
