package si.ijs.urbanite.simserver.travel_demand;

import java.io.*;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.*;
import java.util.AbstractMap.SimpleEntry;
import java.net.URI;
import org.json.simple.JSONArray;
import org.json.simple.JSONObject;
import org.json.simple.parser.JSONParser;
import org.poly2tri.Poly2Tri;
import org.poly2tri.geometry.polygon.Polygon;
import org.poly2tri.geometry.polygon.PolygonPoint;
import org.poly2tri.triangulation.TriangulationPoint;
import org.poly2tri.triangulation.delaunay.*;

public class PolygonPointFinder {
  // return array of arrays -> district with coordinates
  // should probably be a map on the outside
  public Map<String, List<AbstractMap.SimpleEntry<DelaunayTriangle, Double>>> districtAreas;
  public String city;

  public PolygonPointFinder(String city) throws FileNotFoundException {
    this.city = city;

    HashMap<String, Polygon> map = createPolygons();
    districtAreas = new HashMap<>();
    // TODO generate this in advance?
    for (Map.Entry<String, Polygon> district : map.entrySet()) {
      // BASURTO district data is *COMPLETELY* borked
      // last 12 data are wrong
      // System.out.println(district.getKey());
      if (district.getKey().equals("BASURTO")) {
        continue;
      }
      Polygon polygon = district.getValue();
//      System.out.println(district.getValue().getTriangles()+"   ,");
      Poly2Tri.triangulate(polygon);
      List<DelaunayTriangle> triangles = polygon.getTriangles();

      // ? do we need geojson here?
      // JSONObject geoData = new JSONObject();
      // geoData.put("type", "FeatureCollection");
      // JSONArray features = new JSONArray();

      ArrayList<AbstractMap.SimpleEntry<DelaunayTriangle, Double>> triangle_areas = new ArrayList<>();

      // TODO probobaly shouldn't be taking smaller TODO than 1m^2 into account
      // TODO Deusto and Abando are the same
      for (DelaunayTriangle triangle : triangles) {
        double triangleArea = area(triangle.points[0], triangle.points[1], triangle.points[2]);

        triangle_areas.add(new AbstractMap.SimpleEntry<DelaunayTriangle, Double>(triangle, triangleArea));

      }
      districtAreas.put(district.getKey(), triangle_areas);
    }
    districtAreas = aggregateDistrictAreas(districtAreas);
    districtAreas = normalizeDistrictAreas(districtAreas);
  }

  public HashMap<String, Polygon> createPolygons() throws FileNotFoundException {
    HashMap<String, Polygon> polygons = new HashMap<>();

    // TODO download from data storage gtfsShape
    // read shape file
    JSONParser parser = new JSONParser();
    try {
      // TODO add names for all cities, not just Bilbao

      Path path = Paths.get("data", this.city, "before_simulation", "district_shapes.json");
      Object object = parser.parse(new FileReader(path.toFile()));

      JSONArray districts = (JSONArray) object;

      Iterator<JSONObject> iterator = districts.iterator();
      // going over each district entry
      while (iterator.hasNext()) {
        JSONObject obj = iterator.next();
        // "description" is district name
        String district = (String) obj.get("description");
        ArrayList<PolygonPoint> border = new ArrayList<>();
        // initialised x and y to check for duplicate points
        double x = Double.MAX_VALUE, y = Double.MAX_VALUE;
        PolygonPoint prevPoint = new PolygonPoint(x, y);
        for (Object item : (JSONArray) (((JSONObject) obj.get("location")).get("coordinates"))) {
          for (Object coordinate_pair : (JSONArray) item) {
            if (border.size() != 0) {
              // checking the point is duplicate of the previous point
              if (x != (double) ((JSONArray) coordinate_pair).get(0)
                  || y != (double) ((JSONArray) coordinate_pair).get(1)) {
                x = (double) ((JSONArray) coordinate_pair).get(0);
                y = (double) ((JSONArray) coordinate_pair).get(1);
                // checking for duplicates of the starting point
                if (x == border.get(0).getX() && y == border.get(0).getY()) {
                  break;
                }
              }
            }
            x = (double) ((JSONArray) coordinate_pair).get(0);
            y = (double) ((JSONArray) coordinate_pair).get(1);
            PolygonPoint point = new PolygonPoint(x, y);
            if (prevPoint.getX() != point.getX() || prevPoint.getY() != point.getY()) {
              border.add(point);
              prevPoint = point;
            }
          }
          // create the actual polygon object, smallest possible polygon is a
          // triangle with 3 points
          if (border.size() > 2) {
            Polygon polygon = new Polygon(border);
            polygons.put(district, polygon);
            // district is a String name, polygon is a triangulatable object
          }
        }
      }
    } catch (Exception e) {
      e.printStackTrace();
    }

    return polygons;
  }

  // source:
  // https://www.java67.com/2016/10/how-to-calculate-area-of-triangle-in.html
  static double area(TriangulationPoint A, TriangulationPoint B, TriangulationPoint C) {
    double area = (A.getX() * (B.getY() - C.getY()) + B.getX() * (C.getY() - A.getY())
        + C.getX() * (A.getY() - B.getY())) / 2.0;
    return Math.abs(area);
  }

  public void printGeodata(String district, JSONObject geoData) throws IOException {
    File file = new File(String.valueOf(Paths.get("data", this.city, "districts", district, "trig.json")));
    file.getParentFile().mkdirs();
    FileWriter f = new FileWriter(String.valueOf(Paths.get("data", this.city, "districts", district, "trig.json")));
    f.write(geoData.toString());
    f.close();
  }

  public Map<String, List<AbstractMap.SimpleEntry<DelaunayTriangle, Double>>> aggregateDistrictAreas(
      Map<String, List<AbstractMap.SimpleEntry<DelaunayTriangle, Double>>> districtAreas) {

    Map<String, List<AbstractMap.SimpleEntry<DelaunayTriangle, Double>>> newDistrictAreas = new HashMap<String, List<AbstractMap.SimpleEntry<DelaunayTriangle, Double>>>();
    Map<String, String[]> districts = new HashMap<>();

    if (this.city == "bilbao") {
      districts.put("DEUSTO", new String[] { "Deusto" });
      districts.put("URIBARRI", new String[] { "Uríbarri" });
      districts.put("OTXARKOAGA_TXURDINAGA", new String[] { "Otxarkoaga-Txurdinaga" });
      districts.put("BEGONA", new String[] { "Begoña" });
      districts.put("IBAIONDO", new String[] { "Ibaiondo" });
      districts.put("ABANDO", new String[] { "Abando" });
      districts.put("ERREKALDE", new String[] { "D7 NORTE", "D7 SUR" });
      districts.put("BASURTO_ZORROTZA", new String[] { "Basurto-Zorroza" });
    } else if (this.city == "messina") {
      districts.put("I Circoscrizione", new String[] { "I Circoscrizione" });
      districts.put("II Circoscrizione", new String[] { "II Circoscrizione" });
      districts.put("III Circoscrizione", new String[] { "III Circoscrizione" });
      districts.put("IV Circoscrizione", new String[] { "IV Circoscrizione" });
      districts.put("V Circoscrizione", new String[] { "V Circoscrizione" });
      districts.put("VI Circoscrizione", new String[] { "VI Circoscrizione" });
    }
    // how do we aggregate these?
    else if (this.city == "amsterdam") {
      districts.put("Centrum-West DX-01", new String[] { "Centrum-West DX-01" });
      districts.put("Centrum-Oost DX-02", new String[] { "Centrum-Oost DX-02", });
      districts.put("Westerpark DX-03", new String[] { "Westerpark DX-03", });
      districts.put("Bos en Lommer DX-04", new String[] { "Bos en Lommer DX-04", });
      districts.put("Oud-West DX-05", new String[] { "Oud-West DX-05", });
      districts.put("DX-06", new String[] { "DX-06", });
      districts.put("DX-07", new String[] { "DX-07", });
      districts.put("DX-08", new String[] { "DX-08", });
      districts.put("DX-09", new String[] { "DX-09", });
      districts.put("DX-10", new String[] { "DX-10", });
      districts.put("DX-11", new String[] { "DX-11", });
      districts.put("DX-12", new String[] { "DX-12", });
      districts.put("DX-13", new String[] { "DX-13", });
      districts.put("DX-14", new String[] { "DX-14", });
      districts.put("DX-15", new String[] { "DX-15", });
      districts.put("DX-16", new String[] { "DX-16", });
      districts.put("DX-17", new String[] { "DX-17", });
      districts.put("DX-18", new String[] { "DX-18", });
      districts.put("DX-19", new String[] { "DX-19", });
      districts.put("DX-20", new String[] { "DX-20", });
      districts.put("DX-21", new String[] { "DX-21", });
      districts.put("DX-22", new String[] { "DX-22", });
      districts.put("ZX-99", new String[] { "ZX-99", });
    }

    for (Map.Entry<String, String[]> district : districts.entrySet()) {
      ArrayList<AbstractMap.SimpleEntry<DelaunayTriangle, Double>> triangles = new ArrayList<AbstractMap.SimpleEntry<DelaunayTriangle, Double>>();
      for (String subDistrict : district.getValue()) {
        try {
          triangles.addAll(districtAreas.get(subDistrict));
        } catch (Exception e) {
        }
      }

      newDistrictAreas.put(district.getKey(), triangles);
    }

    return newDistrictAreas;
  }

  public List<AbstractMap.SimpleEntry<DelaunayTriangle, Double>> getTriangleAreas(String district_name)
      throws FileNotFoundException {

    return this.districtAreas.get(district_name);
  }

  static Map<String, List<SimpleEntry<DelaunayTriangle, Double>>> normalizeDistrictAreas(
      Map<String, List<SimpleEntry<DelaunayTriangle, Double>>> districtAreas) {
    for (Map.Entry<String, List<SimpleEntry<DelaunayTriangle, Double>>> district : districtAreas.entrySet()) {
      double totalArea = 0;
      for (SimpleEntry<DelaunayTriangle, Double> triangle : district.getValue()) {
        totalArea += triangle.getValue();
      }
      for (SimpleEntry<DelaunayTriangle, Double> triangle : district.getValue()) {
        triangle.setValue(triangle.getValue() / totalArea);
      }
    }

    return districtAreas;
  }

  static DelaunayTriangle getRandomTriangle(List<AbstractMap.SimpleEntry<DelaunayTriangle, Double>> triangle_areas) {
    double random = Math.random();

    int i;
    for (i = 0; random > 0; i++) {
      random -= triangle_areas.get(i).getValue();
    }
    return triangle_areas.get(i - 1).getKey();
  }

  static PolygonPoint getRandomPointWithinTriangle(DelaunayTriangle triangle) {
    double r1 = Math.random();
    double r2 = Math.random();
    double x = (1 - Math.sqrt(r1)) * triangle.points[0].getX() + (Math.sqrt(r1) * (1 - r2)) * triangle.points[1].getX()
        + (Math.sqrt(r1) * r2) * triangle.points[2].getX();
    double y = (1 - Math.sqrt(r1)) * triangle.points[0].getY() + (Math.sqrt(r1) * (1 - r2)) * triangle.points[1].getY()
        + (Math.sqrt(r1) * r2) * triangle.points[2].getY();

    PolygonPoint p = new PolygonPoint(x, y);

    return p;
  }

  public PolygonPoint getRandomPointInDistrict(String district_name) throws FileNotFoundException {

    // TODO store temporary values?
    return getRandomPointWithinTriangle(getRandomTriangle(getTriangleAreas(district_name)));
  }

  public static void main(String[] args) throws IOException {
    PolygonPointFinder f = new PolygonPointFinder("bilbao");
//    System.out.println(f.getRandomPointInDistrict("DEUSTO"));

  }
}
