package si.ijs.urbanite.simserver.travel_demand;

import com.sun.xml.bind.v2.TODO;
import org.matsim.api.core.v01.Coord;
import org.matsim.api.core.v01.Id;
import org.matsim.api.core.v01.Scenario;
import org.matsim.api.core.v01.network.Network;
import org.matsim.api.core.v01.population.*;
import org.matsim.core.api.internal.MatsimWriter;
import org.matsim.core.config.Config;
import org.matsim.core.config.ConfigUtils;
import org.matsim.core.scenario.ScenarioUtils;
import org.matsim.core.utils.collections.ArrayMap;
import org.matsim.core.utils.geometry.CoordinateTransformation;
import org.matsim.core.utils.geometry.transformations.TransformationFactory;
import org.poly2tri.geometry.polygon.PolygonPoint;
import si.ijs.urbanite.simserver.simulation.SimulationWithParamsDTO;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.nio.file.Path;
import java.util.*;
import java.nio.file.Paths;

public class TravelDemandGenerationBilbao {
    // bilbao travel demand

    // global parameter for the od matrices
    public static HashMap<Integer, ArrayList> odm;

    // ------------------------------------------------------------
    // read OD matrices
    // ------------------------------------------------------------
    public static HashMap<Integer, ArrayList> readODM(Integer scale_factor) throws IOException {
        BufferedReader bf = new BufferedReader(new FileReader(
                Paths.get("data", "bilbao", "before_simulation", "ODM_bilbao_districtLevel_codes.csv").toString()));
        HashMap<Integer, ArrayList> data = new HashMap<>();
        String line = "";
        Boolean flag = true;
        while ((line = bf.readLine()) != null) {
            if (flag) {
                flag = false;
            } else {
                String[] a = line.split(",", 5);
                Integer from_loc = Integer.parseInt(a[0]);
                Integer to_loc = Integer.parseInt(a[1]);
                Integer volume = Integer.parseInt(a[2]);
                if (data.containsKey(from_loc)) {
                    ArrayList<Integer> pom = data.get(from_loc);
                    pom.set(to_loc - 1, volume / scale_factor);
                    data.put(from_loc, pom);
                } else {
                    ArrayList<Integer> pom = new ArrayList<>(Collections.nCopies(8, 0));
                    pom.set(to_loc - 1, volume / scale_factor);
                    data.put(from_loc, pom);
                }
            }
        }
        return data;
    }

    // ------------------------------------------------------------
    // take next location to maintain the od volumes
    // ------------------------------------------------------------
    public static Integer getNextLoc(int currentLoc) {
        Random random = new Random();
        int next_loc = 0;
        ArrayList<Integer> to_locations = odm.get(currentLoc);
        int index = random.nextInt(8);
        // check if all to location values are zero
        boolean check = true; // default all are zeros, if there is non zero element... check=true
        for (int i = 0; i < 8; i++) {
            if (to_locations.get(i) != 0)
                check = false;
        }
        if (check) {
            // all elements are zeros, assign random loc
            next_loc = index;
        } else if (to_locations.get(index) != 0) {
            next_loc = index;
            to_locations.set(index, to_locations.get(index) - 1);
            odm.put(currentLoc, to_locations);
        }

        return next_loc + 1;
    }

    public static void generateTravelDemand(String net_id, String plan_id, SimulationWithParamsDTO params) throws IOException {


        ChangeNumberInhabitants.changeNumberInhabitants("bilbao",params.bilbao_districts(),params.bilbao_districts_index());

        Integer scale_facotr = 10; // 10% of the full population

        Random random = new Random();

        Config config = ConfigUtils.loadConfig(Paths.get("data", "bilbao", "original_input", "config.xml").toString());
        Scenario sc = ScenarioUtils.createScenario(config);
        Network network = sc.getNetwork();

        // create empty population
        Population population = sc.getPopulation();
        PopulationFactory populationFactory = population.getFactory();

        // Coordinate transformation
        CoordinateTransformation transformation = TransformationFactory.getCoordinateTransformation(
                TransformationFactory.WGS84, sc.getConfig().global().getCoordinateSystem());

        // define modes
        String car = "car";
        String walk = "walk";
        String pt = "pt";
        String bicycle = "bicycle";

        // define activity type
        String work = "work";
        String leisure = "leisure";
        String shopping = "shopping";
        String education = "education";
        String other = "other";

        // car availability
        String always = "always";
        String never = "never";
        String sometime = "sometime";

        // sex
        String male = "male";
        String female = "female";

        // district:
        // 1 ABANDO
        // 2 BASURTU-ZORROTZA
        // 3 BEGONA
        // 4 DEUSTU
        // 5 ERREKALDE
        // 6 IBAIONDO
        // 7 OTXARKOAGA-TXURDINAGA
        // 8 URIBARRI

        String[] districts = { "ABANDO", "BASURTO_ZORROTZA", "BEGONA", "DEUSTO", "ERREKALDE", "IBAIONDO",
                "OTXARKOAGA_TXURDINAGA", "URIBARRI" };

        double x1 = 43.261389;
        double y1 = -2.927222;
        double x2 = 43.268056;
        double y2 = -2.965556;
        double x3 = 43.258686;
        double y3 = -2.913847;
        double x4 = 43.272222;
        double y4 = -2.945833;
        double x5 = 43.251944;
        double y5 = -2.946111;
        double x6 = 43.2569;
        double y6 = -2.92333;
        double x7 = 43.258333;
        double y7 = -2.902778;
        double x8 = 43.2678;
        double y8 = -2.92306;

        ArrayList<Double> x = new ArrayList<>();
        x.add(x1);
        x.add(x2);
        x.add(x3);
        x.add(x4);
        x.add(x5);
        x.add(x6);
        x.add(x7);
        x.add(x8);

        ArrayList<Double> y = new ArrayList<>();
        y.add(y1);
        y.add(y2);
        y.add(y3);
        y.add(y4);
        y.add(y5);
        y.add(y6);
        y.add(y7);
        y.add(y8);

        ArrayList<String> mode = new ArrayList<>(); // this array's types must be used as the factories from the
                                                    // framework require very specific input type
        mode.add(car);
        mode.add(bicycle);
        mode.add(walk);
        mode.add(pt);

        ArrayList<String> availability = new ArrayList<>();
        availability.add(always);
        availability.add(sometime);
        availability.add(never);

        ArrayList<Object> sex = new ArrayList<>();
        sex.add(male);
        sex.add(female);

        ArrayList<String> activity_first = new ArrayList<>();
        activity_first.add(work);
        activity_first.add(education);
        activity_first.add(leisure); // for retired people

        ArrayList<String> activity_second = new ArrayList<>();
        activity_second.add(leisure);
        activity_second.add(shopping);
        activity_second.add(other);

        // read Origin-Destination Matrices
        odm = readODM(scale_facotr);
//        System.out.println(odm);
        Integer current_loc;
        Integer next_loc; // read from the ODM

        // scale down the population to 10%
        ArrayList<Integer> index = new ArrayList<>();
        for (int i = 0; i < 317695 * scale_facotr / 100; i++) {
            index.add(random.nextInt(317695));
        }

        // regular workers and delivery workers
        int reg_worker = 0;
        int del_worker = 0;

        // read micrdata
        BufferedReader microdata = new BufferedReader(
                new FileReader(Paths.get("data", "bilbao", "before_simulation", "ind_full.csv").toString()));
        String line;
        boolean header = true; // check if we are in the header
        Integer counter = 0; // only select lines chosen by index array


        PolygonPointFinder rPointFinder = new PolygonPointFinder("bilbao");
        PolygonPoint rPoint;

        while ((line = microdata.readLine()) != null) {
            String[] person_attr = line.split(",");
            if (header) {
                header = false;
            } else {
                if (index.contains(counter)) {
                    String ageString = person_attr[1];
                    int age = Integer.parseInt(ageString);
                    String s = person_attr[2];
                    int home = Integer.parseInt(person_attr[3]);

                    // --------------------------------------------------
                    // define car availability
                    // --------------------------------------------------
                    int aw = 0; // car availability
                    if (age < 16) {
                        aw = 2;
                    } else {
                        aw = random.nextInt(2 + 1);
                    }

                    // -------------------------------------------------------
                    // define student, employee, retired
                    // -------------------------------------------------------
                    int rdE;
                    if (age < 18) {
                        rdE = 0; // student
                    } else if (age < 24 && age > 18) {
                        rdE = random.nextInt(2); // worker or student
                    } else if (age > 24 && age < 65) {
                        rdE = 1; // definitely worker
                    } else {
                        rdE = 2; // elderly
                    }

                    int m;
                    if (availability.get(aw).equals("never")) {
                        m = random.nextInt(3) + 1;
                    } else {
                        m = random.nextInt(4);
                    }

                    double blur_loc = -0.01 + (0.01 - (-0.01)) * random.nextDouble(); // in meters
                    Person person = populationFactory.createPerson(Id.createPersonId(person_attr[0]));
                    person.getAttributes().putAttribute("age", age);
                    person.getAttributes().putAttribute("student, employee, retired", rdE);
                    person.getAttributes().putAttribute("sex", s);
                    person.getAttributes().putAttribute("car availability", availability.get(aw));

                    population.addPerson(person);

                    Plan plan = populationFactory.createPlan();

                    // -------------------------------------------------------------------------------------------------------------------
                    // Create the first activity. The first activity start time is different for
                    // students (young, old) | workers | retired
                    // --------------------------------------------------------------------------------------------------------------------
                    int start_time;
                    int end_time;

                    current_loc = home;
                    // -----------------------------------------------------------------
                    // two types of workers: regular and delivery services (by bicycle)
                    // ------------------------------------------------------------------
                    // get random point
                    PolygonPoint rPoint_home = rPointFinder.getRandomPointInDistrict(districts[home - 1]);

                    if (rdE == 0) {
                        // student
                        if (age < 18) {
                            start_time = 7 * 60 * 60 + random.nextInt(7200);

                            Activity activity1 = populationFactory.createActivityFromCoord("home",
                                    transformation.transform(new Coord(y.get(home-1) + blur_loc,
                                            x.get(home-1) + blur_loc)));
                            activity1.setEndTime(start_time - (random.nextInt(30 * 60) + 30 * 60));
                            plan.addActivity(activity1);
                            plan.addLeg(populationFactory.createLeg(mode.get(m)));
                            next_loc = getNextLoc(current_loc);
                            rPoint = rPointFinder.getRandomPointInDistrict(districts[next_loc - 1]);
                            Activity activity2 = populationFactory.createActivityFromCoord("education",
                                    transformation.transform(new Coord(y.get(random.nextInt(8)), x.get(random.nextInt(8)))));
                            activity2.setStartTime(start_time); // start time from 07.00 to 09.00
                            activity2.setEndTime(start_time + random.nextInt(4 * 60 * 60) + 4 * 60 * 60);
                            plan.addActivity(activity2);
                            plan.addLeg(populationFactory.createLeg(mode.get(m)));
                            current_loc = next_loc;
                        } else {
                            // if the person is old student
                            start_time = 7 * 60 * 60 + random.nextInt(7200); // start time of first activity
                            rPoint = rPointFinder.getRandomPointInDistrict(districts[home - 1]);
                            Activity activity1 = populationFactory.createActivityFromCoord("home",
                                    transformation.transform(new Coord(y.get(home-1), x.get(home-1))));
                            activity1.setEndTime(start_time - (random.nextInt(30 * 60) + 30 * 60));
                            plan.addActivity(activity1);
                            plan.addLeg(populationFactory.createLeg(mode.get(m)));
                            next_loc = getNextLoc(current_loc);
                            rPoint = rPointFinder.getRandomPointInDistrict(districts[next_loc - 1]);
                            Activity activity2 = populationFactory.createActivityFromCoord("education",
                                    transformation.transform(new Coord(y.get(random.nextInt(8)), x.get(random.nextInt(8)))));
                            end_time = start_time + random.nextInt(4 * 60 * 60) + 6 * 60 * 60;
                            activity2.setStartTime(start_time); // start time from 07.00 to 09.00
                            activity2.setEndTime(end_time);
                            plan.addActivity(activity2);
                            plan.addLeg(populationFactory.createLeg(mode.get(m)));
                            current_loc = next_loc;

                            // old students have leisure activities before midnight

                            next_loc = getNextLoc(current_loc);
                            rPoint = rPointFinder.getRandomPointInDistrict(districts[next_loc - 1]);
                            Activity act_other = populationFactory.createActivityFromCoord(
                                    activity_second.get(random.nextInt(3)),
                                    transformation.transform(new Coord(y.get(random.nextInt(8)), x.get(random.nextInt(8)))));
                            start_time = end_time + 60 * 60;
                            end_time = start_time + random.nextInt(4 * 60 * 60);
                            act_other.setStartTime(start_time);
                            act_other.setEndTime(end_time);
                            plan.addActivity(act_other);
                            plan.addLeg(populationFactory.createLeg(mode.get(m)));
                            current_loc = next_loc;
                        }
                    } else if (rdE == 1) {
                        // worker, two types of workers: regular and delivery services
                        // delivery services are 20 percents of all workers
                        // System.out.println("delivery workers: "+del_worker+" regular workers:
                        // "+reg_worker);
                        if (del_worker < (reg_worker * 20 / 100)) {
                            del_worker++;
                            // delivery services all day
                            start_time = 8 * 60 * 60 + random.nextInt(2 * 60 * 60);
                            rPoint = rPointFinder.getRandomPointInDistrict(districts[home - 1]);
                            Activity activity1 = populationFactory.createActivityFromCoord("home",
                                    transformation.transform(new Coord(y.get(home-1), x.get(home-1))));
                            activity1.setEndTime(start_time - (random.nextInt(40 * 60) + 40));
                            plan.addActivity(activity1);
                            plan.addLeg(populationFactory.createLeg(mode.get(1))); // bicycle is leg mode
                            for (int i = 0; i < random.nextInt(7) + 5; i++) {
                                next_loc = getNextLoc(current_loc);
                                rPoint = rPointFinder.getRandomPointInDistrict(districts[next_loc - 1]);
                                Activity delivery = populationFactory.createActivityFromCoord("delivery",
                                        transformation.transform(new Coord(y.get(random.nextInt(8)),x.get(random.nextInt(8)))));
                                delivery.setStartTime(start_time);
                                end_time = start_time + random.nextInt(1 * 60 * 60);
                                delivery.setEndTime(end_time);
                                plan.addActivity(delivery);
                                plan.addLeg(populationFactory.createLeg(mode.get(1))); // bicycle is leg mode
                                current_loc = next_loc;
                                start_time = end_time + random.nextInt(1 * 60 * 60) + 30 * 60;

                            }

                        } else {
                            // System.out.println(reg_worker);
                            reg_worker = reg_worker + 1;
                            // regular workers, first activity is work, then leisure/shopping
                            start_time = 6 * 60 * 60 + random.nextInt(3 * 60 * 60);
                            rPoint = rPointFinder.getRandomPointInDistrict(districts[home - 1]);
                            Activity activity1 = populationFactory.createActivityFromCoord("home",
                                    transformation.transform(new Coord(y.get(home-1), x.get(home-1))));
                            activity1.setEndTime(start_time - (random.nextInt(40 * 60) + 40));
                            plan.addActivity(activity1);
                            plan.addLeg(populationFactory.createLeg(mode.get(m)));
                            end_time = start_time + random.nextInt(3 * 60 * 60) + 6 * 60 * 60;
                            next_loc = getNextLoc(current_loc);
                            rPoint = rPointFinder.getRandomPointInDistrict(districts[next_loc - 1]);
                            Activity activity2 = populationFactory.createActivityFromCoord("work",
                                    transformation.transform(new Coord(y.get(random.nextInt(8)), x.get(random.nextInt(8)))));
                            activity2.setStartTime(start_time);
                            activity2.setEndTime(end_time);
                            plan.addActivity(activity2);
                            plan.addLeg(populationFactory.createLeg(mode.get(m)));
                            current_loc = next_loc;

                            // after work can have two other activities (shopping, leisure, other)
                            for (int i = 0; i < random.nextInt(2) + 1; i++) {
                                next_loc = getNextLoc(current_loc);
                                start_time = end_time + random.nextInt(1 * 60 * 60) + 30 * 60;
                                end_time = start_time + random.nextInt(1 * 60 * 60);
                                rPoint = rPointFinder.getRandomPointInDistrict(districts[next_loc - 1]);
                                Activity act_other = populationFactory.createActivityFromCoord(
                                        activity_second.get(random.nextInt(3)),
                                        transformation.transform(new Coord(y.get(random.nextInt(8)), x.get(random.nextInt(8)))));
                                act_other.setStartTime(start_time);
                                act_other.setEndTime(end_time);
                                plan.addActivity(act_other);
                                plan.addLeg(populationFactory.createLeg(mode.get(m)));
                                current_loc = next_loc;
                            }

                        }

                    } else {
                        // retired, can have up to 3 activities
                        start_time = 9 * 60 * 60 + random.nextInt(2 * 60 * 60);
                        rPoint = rPointFinder.getRandomPointInDistrict(districts[home - 1]);
                        Activity activity1 = populationFactory.createActivityFromCoord("home",
                                transformation.transform(new Coord(y.get(home-1), x.get(home-1))));
                        activity1.setEndTime(start_time - random.nextInt(40 * 60));
                        plan.addActivity(activity1);
                        plan.addLeg(populationFactory.createLeg(mode.get(m)));
                        for (int i = 0; i < random.nextInt(3) + 1; i++) {
                            next_loc = getNextLoc(current_loc);
                            end_time = start_time + random.nextInt(40 * 60) + 40 * 60;
                            rPoint = rPointFinder.getRandomPointInDistrict(districts[next_loc - 1]);
                            Activity act_other = populationFactory.createActivityFromCoord(
                                    activity_second.get(random.nextInt(3)),
                                    transformation.transform(new Coord(y.get(random.nextInt(8)), x.get(random.nextInt(8)))));
                            act_other.setStartTime(start_time);
                            act_other.setEndTime(end_time);
                            plan.addActivity(act_other);
                            plan.addLeg(populationFactory.createLeg(mode.get(m)));
                            current_loc = next_loc;
                            start_time = end_time + random.nextInt(2 * 60 * 60) + 30 * 60;
                        }
                    }
//                    rPoint = rPointFinder.getRandomPointInDistrict(districts[home - 1]);
                    Activity activity3 = populationFactory.createActivityFromCoord("home",
                            transformation.transform(new Coord(y.get(home-1), x.get(home-1))));
                    plan.addActivity(activity3);
                    person.addPlan(plan);
                }
                counter++;
            }
        }
        // TODO: make save in specified networks, net_id, plans,plan_id folder
        MatsimWriter popWriter = new PopulationWriter(population, network); // We finally add the person to file
        Path path = Paths.get("data", "bilbao", "original_input", "networks", net_id, "plans", plan_id, "plans.xml");
        File file = new File(String.valueOf(path));
        file.getParentFile().mkdirs();
        popWriter.write(String.valueOf(path));

        microdata.close();
//        System.out.println(odm);
    }

    public static void main(String[] args) throws IOException {
//        generateTravelDemand("1", "1");
    }
}
