package si.ijs.urbanite.simserver.travel_demand;

import org.matsim.api.core.v01.Coord;
import org.matsim.api.core.v01.Id;
import org.matsim.api.core.v01.Scenario;
import org.matsim.api.core.v01.network.Network;
import org.matsim.api.core.v01.population.*;
import org.matsim.core.api.internal.MatsimWriter;
import org.matsim.core.config.Config;
import org.matsim.core.config.ConfigUtils;
import org.matsim.core.scenario.ScenarioUtils;
import org.matsim.core.utils.geometry.CoordinateTransformation;
import org.matsim.core.utils.geometry.transformations.TransformationFactory;
import org.poly2tri.geometry.polygon.PolygonPoint;
import si.ijs.urbanite.simserver.simulation.SimulationWithParamsDTO;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.Random;

public class TravelDemandGenerationMessina {
    public static void generateTravelDemand(String net_id, String plan_id, SimulationWithParamsDTO paramsDTO) throws IOException {

        ChangeNumberInhabitants.changeNumberInhabitants("messina",paramsDTO.messina_districts(),paramsDTO.messina_districts_index());
        Integer scale_facotr = 10; // 10% of the full population

        Random random = new Random();

        Config config = ConfigUtils.loadConfig(Paths.get("data", "messina", "original_input", "config.xml").toString());
        Scenario sc = ScenarioUtils.createScenario(config);
        Network network = sc.getNetwork();

        // create empty population
        Population population = sc.getPopulation();
        PopulationFactory populationFactory = population.getFactory();

        // Coordinate transformation
        CoordinateTransformation transformation = TransformationFactory.getCoordinateTransformation(
                TransformationFactory.WGS84, sc.getConfig().global().getCoordinateSystem());

        // define modes
        String car = "car";
        String walk = "walk";
        String pt = "pt";
        String bicycle = "bicycle";

        // define activity type
        String work = "work";
        String leisure = "leisure";
        String shopping = "shopping";
        String education = "education";
        String other = "other";

        // car availability
        String always = "always";
        String never = "never";
        String sometime = "sometime";

        // sex
        String male = "male";
        String female = "female";

        // district:
        // 1 I Circoscrizione
        // 2 II Circoscrizione
        // 3 III Circoscrizione
        // 4 IV Circoscrizione
        // 5 V Circoscrizione
        // 6 VI Circoscrizione

        String[] districts = { "I Circoscrizione", "II Circoscrizione", "III Circoscrizione", "IV Circoscrizione",
                "V Circoscrizione", "VI Circoscrizione" };

        double x1 = 38.11641539382573;
        double y1 = 15.474272674870525;
        double x2 = 38.15940041848691;
        double y2 = 15.5023483805018;
        double x3 = 38.18775489582801;
        double y3 = 15.515656285942306;
        double x4 = 38.19566030172442;
        double y4 = 15.546392315926388;
        double x5 = 38.22230896236674;
        double y5 = 15.534661040744489;
        double x6 = 38.248885617480305;
        double y6 = 15.545763920340907;

        ArrayList<Double> x = new ArrayList<>();
        x.add(x1);
        x.add(x2);
        x.add(x3);
        x.add(x4);
        x.add(x5);
        x.add(x6);

        ArrayList<Double> y = new ArrayList<>();
        y.add(y1);
        y.add(y2);
        y.add(y3);
        y.add(y4);
        y.add(y5);
        y.add(y6);

        ArrayList<String> mode = new ArrayList<>(); // this array's types must be used as the factories from the
                                                    // framework require very specific input type
        mode.add(car);
        mode.add(bicycle);
        mode.add(walk);
        mode.add(pt);

        ArrayList<String> availability = new ArrayList<>();
        availability.add(always);
        availability.add(sometime);
        availability.add(never);

        ArrayList<Object> sex = new ArrayList<>();
        sex.add(male);
        sex.add(female);

        ArrayList<String> activity_first = new ArrayList<>();
        activity_first.add(work);
        activity_first.add(education);
        activity_first.add(leisure); // for retired people

        ArrayList<String> activity_second = new ArrayList<>();
        activity_second.add(leisure);
        activity_second.add(shopping);
        activity_second.add(other);

        // scale down the population to 10%
        ArrayList<Integer> index = new ArrayList<>();
        for (int i = 0; i < 492482 * scale_facotr / 100; i++) {
            index.add(random.nextInt(492482));
        }

        // read micrdata
        BufferedReader microdata = new BufferedReader(
                new FileReader(Paths.get("data", "messina", "before_simulation", "ind_full.csv").toString()));
        String line;
        boolean header = true; // check if we are in the header
        Integer counter = 0; // only select lines chosen by index array

//        PolygonPointFinder rPointFinder = new PolygonPointFinder("messina");
        PolygonPoint rPoint;

        while ((line = microdata.readLine()) != null) {
            String[] person_attr = line.split(",");
            if (header) {
                header = false;
            } else {
                if (index.contains(counter)) {
                    String ageString = person_attr[1];
                    int age = Integer.parseInt(ageString);
                    String s = person_attr[2];
                    int home = Integer.parseInt(person_attr[3]);

                    // Age groups:
                    //
                    // 0-19 students
                    // 20-44 workers
                    // 45-64 workers
                    // 65+ retired

                    // --------------------------------------------------
                    // define car availability
                    // --------------------------------------------------
                    int aw = 0; // car availability
                    if (age == 1) {
                        aw = 2;
                    } else {
                        aw = random.nextInt(2 + 1);
                    }

                    // -------------------------------------------------------
                    // define student, employee, retired
                    // -------------------------------------------------------
                    int rdE;
                    if (age == 1) {
                        rdE = 0; // student
                    } else if (age == 2 || age == 3) {
                        rdE = 1; // worker or student
                    } else {
                        rdE = 2; // elderly
                    }

                    int m;
                    if (availability.get(aw).equals("never")) {
                        m = random.nextInt(3) + 1;
                    } else {
                        m = random.nextInt(4);
                    }

                    double blur_loc = -0.01 + (0.01 - (-0.01)) * random.nextDouble(); // in meters
                    Person person = populationFactory.createPerson(Id.createPersonId(person_attr[0]));
                    person.getAttributes().putAttribute("age", age);
                    person.getAttributes().putAttribute("student, employee, retired", rdE);
                    person.getAttributes().putAttribute("sex", s);
                    person.getAttributes().putAttribute("car availability", availability.get(aw));

                    population.addPerson(person);

                    Plan plan = populationFactory.createPlan();

                    // -------------------------------------------------------------------------------------------------------------------
                    // Create the first activity. The first activity start time is different for
                    // students (young, old) | workers | retired
                    // --------------------------------------------------------------------------------------------------------------------
                    int start_time;
                    int end_time;

                    // -----------------------------------------------------------------
                    // two types of workers: regular and delivery services (by bicycle)
                    // ------------------------------------------------------------------

                    if (rdE == 0) {
                        // student
                        start_time = 7 * 60 * 60 + random.nextInt(7200);
//                        rPoint = rPointFinder.getRandomPointInDistrict(districts[home - 1]);
                        Activity activity1 = populationFactory.createActivityFromCoord("home",
                                transformation.transform(new Coord(y.get(random.nextInt(6)) + blur_loc,
                                        x.get(random.nextInt(6)) + blur_loc)));
                        activity1.setEndTime(start_time - (random.nextInt(30 * 60) + 30 * 60));
                        plan.addActivity(activity1);
                        plan.addLeg(populationFactory.createLeg(mode.get(m)));
//                        rPoint = rPointFinder.getRandomPointInDistrict(districts[random.nextInt(6)]);
                        Activity activity2 = populationFactory.createActivityFromCoord("education",
                                transformation.transform(new Coord(y.get(random.nextInt(6)) + blur_loc,
                                        x.get(random.nextInt(6)) + blur_loc)));
                        activity2.setStartTime(start_time); // start time from 07.00 to 09.00
                        activity2.setEndTime(start_time + random.nextInt(4 * 60 * 60) + 4 * 60 * 60);
                        plan.addActivity(activity2);
                        plan.addLeg(populationFactory.createLeg(mode.get(m)));

                    } else if (rdE == 1) {
                        // System.out.println(reg_worker);

                        // regular workers, first activity is work, then leisure/shopping
                        start_time = 6 * 60 * 60 + random.nextInt(3 * 60 * 60);
//                        rPoint = rPointFinder.getRandomPointInDistrict(districts[home - 1]);
                        Activity activity1 = populationFactory.createActivityFromCoord("home",
                                transformation.transform(new Coord(y.get(random.nextInt(6)) + blur_loc,
                                        x.get(random.nextInt(6)) + blur_loc)));
                        activity1.setEndTime(start_time - (random.nextInt(40 * 60) + 40));
                        plan.addActivity(activity1);
                        plan.addLeg(populationFactory.createLeg(mode.get(m)));
                        end_time = start_time + random.nextInt(3 * 60 * 60) + 6 * 60 * 60;

//                        rPoint = rPointFinder.getRandomPointInDistrict(districts[random.nextInt(6)]);
                        Activity activity2 = populationFactory.createActivityFromCoord("work",
                                transformation.transform(new Coord(y.get(random.nextInt(6)) + blur_loc,
                                        x.get(random.nextInt(6)) + blur_loc)));
                        activity2.setStartTime(start_time);
                        activity2.setEndTime(end_time);
                        plan.addActivity(activity2);
                        plan.addLeg(populationFactory.createLeg(mode.get(m)));

                        // after work can have two other activities (shopping, leisure, other)
                        for (int i = 0; i < random.nextInt(2) + 1; i++) {

                            start_time = end_time + random.nextInt(1 * 60 * 60) + 30 * 60;
                            end_time = start_time + random.nextInt(1 * 60 * 60);
//                            rPoint = rPointFinder.getRandomPointInDistrict(districts[random.nextInt(6)]);
                            Activity act_other = populationFactory.createActivityFromCoord(
                                    activity_second.get(random.nextInt(3)),
                                    transformation.transform(new Coord(y.get(random.nextInt(6)) + blur_loc,
                                            x.get(random.nextInt(6)) + blur_loc)));
                            act_other.setStartTime(start_time);
                            act_other.setEndTime(end_time);
                            plan.addActivity(act_other);
                            plan.addLeg(populationFactory.createLeg(mode.get(m)));

                        }

                    } else {
                        // retired, can have up to 3 activities
                        start_time = 9 * 60 * 60 + random.nextInt(2 * 60 * 60);
//                        rPoint = rPointFinder.getRandomPointInDistrict(districts[home - 1]);
                        Activity activity1 = populationFactory.createActivityFromCoord("home",
                                transformation.transform(new Coord(y.get(random.nextInt(6)) + blur_loc,
                                        x.get(random.nextInt(6)) + blur_loc)));
                        activity1.setEndTime(start_time - random.nextInt(40 * 60));
                        plan.addActivity(activity1);
                        plan.addLeg(populationFactory.createLeg(mode.get(m)));
                        for (int i = 0; i < random.nextInt(3) + 1; i++) {

                            end_time = start_time + random.nextInt(40 * 60) + 40 * 60;
//                            rPoint = rPointFinder.getRandomPointInDistrict(districts[random.nextInt(6)]);
                            Activity act_other = populationFactory.createActivityFromCoord(
                                    activity_second.get(random.nextInt(3)),
                                    transformation.transform(new Coord(y.get(random.nextInt(6)) + blur_loc,
                                            x.get(random.nextInt(6)) + blur_loc)));
                            act_other.setStartTime(start_time);
                            act_other.setEndTime(end_time);
                            plan.addActivity(act_other);
                            plan.addLeg(populationFactory.createLeg(mode.get(m)));

                            start_time = end_time + random.nextInt(2 * 60 * 60) + 30 * 60;
                        }
                    }
//                    rPoint = rPointFinder.getRandomPointInDistrict(districts[home - 1]);
                    Activity activity3 = populationFactory.createActivityFromCoord("home",
                            transformation.transform(new Coord(y.get(random.nextInt(6)) + blur_loc,
                                    x.get(random.nextInt(6)) + blur_loc)));
                    plan.addActivity(activity3);
                    person.addPlan(plan);
                }
                counter++;
            }
        }
        // TODO: make save in specified networks, net_id, plans,plan_id folder
        MatsimWriter popWriter = new PopulationWriter(population, network); // We finally add the person to file
        popWriter
                .write(Paths.get("data", "messina", "original_input", "networks", net_id, "plans", plan_id, "plans.xml")
                        .toString());

        microdata.close();

    }

    public static void main(String[] args) throws IOException {
//        generateTravelDemand("2","2");
    }
}
