package si.ijs.urbanite.simserver.travel_demand;

import org.matsim.api.core.v01.Coord;
import org.matsim.api.core.v01.Id;
import org.matsim.api.core.v01.Scenario;
import org.matsim.api.core.v01.network.Network;
import org.matsim.api.core.v01.population.*;
import org.matsim.core.api.internal.MatsimWriter;
import org.matsim.core.config.Config;
import org.matsim.core.config.ConfigUtils;
import org.matsim.core.scenario.ScenarioUtils;
import org.matsim.core.utils.geometry.CoordinateTransformation;
import org.matsim.core.utils.geometry.transformations.TransformationFactory;
import si.ijs.urbanite.simserver.simulation.SimulationWithParamsDTO;

import javax.persistence.criteria.CriteriaBuilder;
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.lang.reflect.Array;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Random;
import java.util.UUID;

public class TravelDmenadAmsterdamCyclistsCut {
    public static void generateTravelDemand(String net_id, String plan_id, int scale_factor, boolean amst_nord, SimulationWithParamsDTO paramsDTO) throws IOException {
//        Integer scale_facotr = 1; // 10% of the full population

        ChangeNumberInhabitants.changeNumberInhabitants("amsterdam",paramsDTO.amsterdam_districts(),paramsDTO.amsterdam_index());
        Random random = new Random();

        Config config = ConfigUtils.loadConfig(Paths.get("data", "amsterdam", "original_input", "config.xml").toString());
        Scenario sc = ScenarioUtils.createScenario(config);
        Network network = sc.getNetwork();

        //create empty population
        Population population = sc.getPopulation();
        PopulationFactory populationFactory = population.getFactory();

        //Coordinate transformation
        CoordinateTransformation transformation = TransformationFactory.getCoordinateTransformation(TransformationFactory.WGS84, sc.getConfig().global().getCoordinateSystem());

        // define modes
        String car = "car";
        String walk = "walk";
        String pt = "pt";
        String bicycle_slow = "bicycle_slow";
        String bicycle_medoum = "bicycle_medium";
        String bicycle_fast = "bicycle_fast";

        // define activity type
        String work = "work";
        String leisure = "leisure";
        String shopping = "shopping";
        String education = "education";
        String other = "other";

        //car availability
        String always = "always";
        String never = "never";
        String sometime = "sometime";

        //sex
        String male = "male";
        String female = "female";

        //        Since the demographic statistics are not available for every age, population is devided into age groups like the following:
//        1. 0-3
//        2. 4-12
//        3. 13-17
//        4. 18-22
//        5. 23-24
//        6. 25-49
//        7. 50-64
//        8. 65+
//


        // districts
        // Centrum-West DX-01
        double x1 = 52.370804;
        double y1 = 4.898083;
        // Centrum-Oost DX-02
        double x2 = 52.35902692446838;
        double y2 = 4.9103086384911965;
        //Westerpark DX-03 west
        double x3 = 52.386122396154434;
        double y3 = 4.8765117171799925;
        // Bos en Lommer DX-04 west
        double x4 = 52.38102829415163;
        double y4 = 4.853871530049244;
        //Oud-West DX-05 west
        double x5 = 52.36289731799234;
        double y5 = 4.870659013290066;
//        //DX-06 F Nieuw-West
//        double x6 = 52.37865794155335;
//        double y6 = 4.800440667295913;
//        //DX-07 F Nieuw-West
//        double x7 = 52.36055591396814;
//        double y7 = 4.7876977147664945;
//        //DX-08 F Nieuw-West
//        double x8 = 52.33854049291761;
//        double y8 = 4.7988705585318066;
        //DX-09 F Nieuw-West
//        double x9 = 52.35642547693459;
//        double y9 = 4.826539933602014;
        //DX-10 K Zuid
        double x10 = 52.35299538767255;
        double y10 = 4.882491757622325;
        //DX-11 K Zuid
        double x11 = 52.33845311216292;
        double y11 = 4.873275245213914;
        //DX-12 K Zuid
        double x12 = 52.34528119902544;
        double y12 = 4.896996645324244;
        //DX-13 M Oost
        double x13 = 52.36007397641167;
        double y13 = 4.9258642544909605;
        //DX-14 M Oost
        double x14 = 52.36410146745712;
        double y14 = 4.939939604131165;
        //DX-15 M Oost
        double x15 = 52.35098369377154;
        double y15 = 4.934635553458878;
        //DX-16 M Oost
//        double x16 = 52.35505351117191;
//        double y16 = 4.997988011182878;
        //DX-17 nord
        double x17 = 52.37312809866058;
        double y17 = 4.8673491207482815;
        //DX-18 nord
        double x18 = 52.396485441593065;
        double y18 = 4.936605520359885;
        //DX-19 nord
        double x19 = 52.396485441593065;
        double y19 = 4.936605520359885;
//        //DX-20 T Zuidoost
//        double x20 = 52.30809369581694;
//        double y20 = 4.977387729009753;
//        //DX-21 T Zuidoost
//        double x21 = 52.3182408922999;
//        double y21 = 4.975551915046268;
//        //DX-22 T Zuidoost
//        double x22 = 52.30657719078675;
//        double y22 = 4.993782901229291;
        //ZX-99 ZX99 Westpoort
//        double x23 = 52.40676337252295;
//        double y23 = 4.795437778352777;

        ArrayList<Double> x = new ArrayList<>();
        x.add(x1);
        x.add(x2);
        x.add(x3);
        x.add(x4);
        x.add(x5);
//        x.add(x6);
//        x.add(x7);
//        x.add(x8);
//        x.add(x9);
        x.add(x10);
        x.add(x11);
        x.add(x12);
        x.add(x13);
        x.add(x14);
        x.add(x15);
//        x.add(x16);
        x.add(x17);
        x.add(x18);
        x.add(x19);
//        x.add(x20);
//        x.add(x21);
//        x.add(x22);
//        x.add(x23);


        ArrayList<Double> y = new ArrayList<>();
        y.add(y1);
        y.add(y2);
        y.add(y3);
        y.add(y4);
        y.add(y5);
//        y.add(y6);
//        y.add(y7);
//        y.add(y8);
//        y.add(y9);
        y.add(y10);
        y.add(y11);
        y.add(y12);
        y.add(y13);
        y.add(y14);
        y.add(y15);
//        y.add(y16);
        y.add(y17);
        y.add(y18);
        y.add(y19);
//        y.add(y20);
//        y.add(y21);
//        y.add(y22);
//        y.add(y23);

        ArrayList<String> mode = new ArrayList<>(); // this array's types must be used as the factories from the framework require very specific input type
        mode.add(car);
        mode.add(bicycle_slow);
        mode.add(bicycle_medoum);
        mode.add(bicycle_fast);
        mode.add(walk);
//        mode.add(pt);

        ArrayList<String> availability = new ArrayList<>();
        availability.add(always);
        availability.add(sometime);
        availability.add(never);

        ArrayList<Object> sex = new ArrayList<>();
        sex.add(male);
        sex.add(female);

        ArrayList<String> activity_first = new ArrayList<>();
        activity_first.add(work);
        activity_first.add(education);
        activity_first.add(leisure); //for retired people

        ArrayList<String> activity_second = new ArrayList<>();
        activity_second.add(leisure);
        activity_second.add(shopping);
        activity_second.add(other);

        ArrayList<Integer> excluded_dist = new ArrayList<>(Arrays.asList(6,7,8,9,16,20,21,22,23));

        //scale down the population to 10%
        ArrayList<Integer> index = new ArrayList<>();
        for (int i = 0; i < 569235 *scale_factor/100; i++) {
            index.add(random.nextInt(569235));
        }

        //regular workers and delivery workers
        int reg_worker = 0;
        int del_worker = 0;

        //read micrdata
        BufferedReader microdata = new BufferedReader(new FileReader(Paths.get("data", "amsterdam", "before_simulation", "ind_full.csv").toString()));
        String line;
        boolean header = true; //check if we are in the header
        Integer counter = 0; // only select lines chosen by index array
        while ((line = microdata.readLine()) != null) {
            String[] person_attr = line.split(",");
            if (header) {
                header = false;
            } else {
                if (index.contains(counter)) {
                    String ageString = person_attr[1];
                    int age = Integer.parseInt(ageString);
                    String s = person_attr[2];
                    int home = Integer.parseInt(person_attr[3]);

                        //--------------------------------------------------
                        // define car availability
                        //--------------------------------------------------
                        int aw = 2; //car availability
                        //age groups
                        if (age < 4) {
                            aw = 2;
                        } else {
                            aw = random.nextInt(3);
                        }


                        //-------------------------------------------------------
                        // define student, employee, retired
                        //-------------------------------------------------------
                        int rdE;
                        if (age < 4) {
                            rdE = 0; //student
                        } else if (age == 4 || age == 5) {
                            rdE = random.nextInt(2); // worker or student
                        } else if (age > 5 && age < 8) {
                            rdE = 1; //definitely worker
                        } else {
                            rdE = 2; // elderly
                        }


                        int m;
                        // car = 0 ; bicycle = 1 ; walk = 2 ;
                        if (availability.get(aw).equals("never")) {
                            m = random.nextInt(2) + 1;
                            if (m == 1) {
                                if (age < 4 || age > 7)
                                    m = 1; //slow rider
                                else
                                    m = random.nextInt(2) + 2;

                            } else if (m == 2)
                                m = 4;

                        } else {
                            m = random.nextInt(3);
                            if (m == 1) {
                                if (age < 4 || age > 7)
                                    m = 1; //slow rider
                                else
                                    m = random.nextInt(2) + 2;
                            } else if (m == 2)
                                m = 4;

                        }


                        double blur_loc = -0.005 + (0.005 - (-0.005)) * random.nextDouble(); //in meters

                        Person person = populationFactory.createPerson(Id.createPersonId(UUID.randomUUID().toString()));
                        person.getAttributes().putAttribute("age", age);
                        person.getAttributes().putAttribute("student, employee, retired", rdE);
                        person.getAttributes().putAttribute("sex", s);
                        person.getAttributes().putAttribute("car availability", availability.get(aw));

                        population.addPerson(person);

                        Plan plan = populationFactory.createPlan();

                        //-------------------------------------------------------------------------------------------------------------------
                        // Create the first activity. The first activity start time is different for students (young, old) | workers | retired
                        //--------------------------------------------------------------------------------------------------------------------
                        int start_time;
                        int end_time;


                        //-----------------------------------------------------------------
                        //two types of workers: regular and delivery services (by bicycle)
                        //------------------------------------------------------------------
                        if (rdE == 0) {
                            //student
                            if (age < 4) {
                                start_time = 7 * 60 * 60 + random.nextInt(7200);
                                Activity activity1 = populationFactory.createActivityFromCoord("home", transformation.transform(new Coord(y.get(home - 1) + blur_loc, x.get(home - 1) + blur_loc)));
                                activity1.setEndTime(start_time - (random.nextInt(30 * 60) + 30 * 60));
                                plan.addActivity(activity1);
                                plan.addLeg(populationFactory.createLeg(mode.get(m)));

                                Activity activity2 = populationFactory.createActivityFromCoord("education", transformation.transform(new Coord(y.get(random.nextInt(x.size())) + blur_loc, x.get(random.nextInt(y.size())) + blur_loc)));
                                activity2.setStartTime(start_time); // start time from 07.00 to 09.00
                                activity2.setEndTime(start_time + random.nextInt(4 * 60 * 60) + 4 * 60 * 60);
                                plan.addActivity(activity2);
                                plan.addLeg(populationFactory.createLeg(mode.get(m)));

                            } else {
                                //if the person is old student
                                start_time = 7 * 60 * 60 + random.nextInt(7200); //start time of first activity
                                Activity activity1 = populationFactory.createActivityFromCoord("home", transformation.transform(new Coord(y.get(home - 1) + blur_loc, x.get(home - 1) + blur_loc)));
                                activity1.setEndTime(start_time - (random.nextInt(30 * 60) + 30 * 60));
                                plan.addActivity(activity1);
                                plan.addLeg(populationFactory.createLeg(mode.get(m)));

                                Activity activity2 = populationFactory.createActivityFromCoord("education", transformation.transform(new Coord(y.get(random.nextInt(x.size())) + blur_loc, x.get(random.nextInt(y.size())) + blur_loc)));
                                end_time = start_time + random.nextInt(4 * 60 * 60) + 6 * 60 * 60;
                                activity2.setStartTime(start_time); // start time from 07.00 to 09.00
                                activity2.setEndTime(end_time);
                                plan.addActivity(activity2);
                                plan.addLeg(populationFactory.createLeg(mode.get(m)));


                                // old students have leisure activities before midnight


                                Activity act_other = populationFactory.createActivityFromCoord(activity_second.get(random.nextInt(3)), transformation.transform(new Coord(y.get(random.nextInt(x.size())) + blur_loc, x.get(random.nextInt(y.size())) + blur_loc)));
                                start_time = end_time + 60 * 60;
                                end_time = start_time + random.nextInt(4 * 60 * 60);
                                act_other.setStartTime(start_time);
                                act_other.setEndTime(end_time);
                                plan.addActivity(act_other);
                                plan.addLeg(populationFactory.createLeg(mode.get(m)));

                            }
                        } else if (rdE == 1) {
                            //System.out.println(reg_worker);
                            reg_worker = reg_worker + 1;
                            // regular workers, first activity is work, then leisure/shopping
                            start_time = 6 * 60 * 60 + random.nextInt(3 * 60 * 60);
                            Activity activity1 = populationFactory.createActivityFromCoord("home", transformation.transform(new Coord(y.get(home - 1) + blur_loc, x.get(home - 1) + blur_loc)));
                            activity1.setEndTime(start_time - (random.nextInt(40 * 60) + 40));
                            plan.addActivity(activity1);
                            plan.addLeg(populationFactory.createLeg(mode.get(m)));
                            end_time = start_time + random.nextInt(3 * 60 * 60) + 6 * 60 * 60;
                            Activity activity2 = populationFactory.createActivityFromCoord("work", transformation.transform(new Coord(y.get(random.nextInt(x.size())) + blur_loc, x.get(random.nextInt(y.size())) + blur_loc)));
                            activity2.setStartTime(start_time);
                            activity2.setEndTime(end_time);
                            plan.addActivity(activity2);
                            plan.addLeg(populationFactory.createLeg(mode.get(m)));


                            //after work can have two other activities (shopping, leisure, other)
                            for (int i = 0; i < random.nextInt(2) + 1; i++) {
                                start_time = end_time + random.nextInt(1 * 60 * 60) + 30 * 60;
                                end_time = start_time + random.nextInt(1 * 60 * 60);
                                Activity act_other = populationFactory.createActivityFromCoord(activity_second.get(random.nextInt(3)), transformation.transform(new Coord(y.get(random.nextInt(x.size())) + blur_loc, x.get(random.nextInt(y.size())) + blur_loc)));
                                act_other.setStartTime(start_time);
                                act_other.setEndTime(end_time);
                                plan.addActivity(act_other);
                                plan.addLeg(populationFactory.createLeg(mode.get(m)));
                            }


                        } else {
                            //retired, can have up to 3 activities
                            start_time = 9 * 60 * 60 + random.nextInt(2 * 60 * 60);
                            Activity activity1 = populationFactory.createActivityFromCoord("home", transformation.transform(new Coord(y.get(home - 1) + blur_loc, x.get(home - 1) + blur_loc)));
                            activity1.setEndTime(start_time - random.nextInt(40 * 60));
                            plan.addActivity(activity1);
                            plan.addLeg(populationFactory.createLeg(mode.get(m)));
                            for (int i = 0; i < random.nextInt(3) + 1; i++) {
                                end_time = start_time + random.nextInt(40 * 60) + 40 * 60;
                                Activity act_other = populationFactory.createActivityFromCoord(activity_second.get(random.nextInt(3)), transformation.transform(new Coord(y.get(random.nextInt(x.size())) + blur_loc, x.get(random.nextInt(y.size())) + blur_loc)));
                                act_other.setStartTime(start_time);
                                act_other.setEndTime(end_time);
                                plan.addActivity(act_other);
                                plan.addLeg(populationFactory.createLeg(mode.get(m)));
                                start_time = end_time + random.nextInt(2 * 60 * 60) + 30 * 60;
                            }
                        }
                        Activity activity3 = populationFactory.createActivityFromCoord("home", transformation.transform(new Coord(y.get(home - 1) + blur_loc, x.get(home - 1) + blur_loc)));
                        plan.addActivity(activity3);
                        person.addPlan(plan);

                }
                counter++;
            }
        }


        if (amst_nord){
            //ad 17000 inhabitants in nord
            for (int k=0;k<17000*scale_factor/100;k++){
                int age = random.nextInt(8)+1;
                int aw = 2; //car availability
                //age groups
                if (age < 4) {
                    aw = 2;
                } else {
                    aw = random.nextInt(3);
                }


                //-------------------------------------------------------
                // define student, employee, retired
                //-------------------------------------------------------
                int rdE;
                if (age < 4) {
                    rdE = 0; //student
                } else if (age ==4 || age ==5) {
                    rdE = random.nextInt(2); // worker or student
                } else if (age > 5 && age < 8) {
                    rdE = 1; //definitely worker
                } else {
                    rdE = 2; // elderly
                }


                int m;
                // car = 0 ; bicycle = 1 ; walk = 2 ;
                if (availability.get(aw).equals("never")) {
                    m = random.nextInt(2) + 1;
                    if (m == 1)
                    {
                        if (age<4 || age>7)
                            m = 1; //slow rider
                        else
                            m = random.nextInt(2)+2;

                    }
                    else if (m == 2)
                        m = 4;

                } else {
                    m = random.nextInt(3);
                    if (m == 1)
                    {
                        if (age<4 || age>7)
                            m = 1; //slow rider
                        else
                            m = random.nextInt(2)+2;
                    }
                    else if(m==2)
                        m = 4;

                }


                double blur_loc = -0.005 + (0.005 - (-0.005)) * random.nextDouble(); //in meters

                Person person = populationFactory.createPerson(Id.createPersonId(UUID.randomUUID().toString()));
                person.getAttributes().putAttribute("nord person",1);
                person.getAttributes().putAttribute("age", age);
                person.getAttributes().putAttribute("student, employee, retired", rdE);
                person.getAttributes().putAttribute("car availability", availability.get(aw));

                population.addPerson(person);

                Plan plan = populationFactory.createPlan();
                int home = random.nextInt(3)+11;
                //-------------------------------------------------------------------------------------------------------------------
                // Create the first activity. The first activity start time is different for students (young, old) | workers | retired
                //--------------------------------------------------------------------------------------------------------------------
                int start_time;
                int end_time;


                //-----------------------------------------------------------------
                //two types of workers: regular and delivery services (by bicycle)
                //------------------------------------------------------------------
                if (rdE == 0) {
                    //student
                    if (age < 4) {
                        start_time = 7 * 60 * 60 + random.nextInt(7200);
                        Activity activity1 = populationFactory.createActivityFromCoord("home", transformation.transform(new Coord(y.get(home - 1) + blur_loc, x.get(home - 1) + blur_loc)));
                        activity1.setEndTime(start_time - (random.nextInt(30 * 60) + 30 * 60));
                        plan.addActivity(activity1);
                        plan.addLeg(populationFactory.createLeg(mode.get(m)));

                        Activity activity2 = populationFactory.createActivityFromCoord("education", transformation.transform(new Coord(y.get(random.nextInt(x.size()) ) + blur_loc, x.get(random.nextInt(y.size()) ) + blur_loc)));
                        activity2.setStartTime(start_time); // start time from 07.00 to 09.00
                        activity2.setEndTime(start_time + random.nextInt(4 * 60 * 60) + 4 * 60 * 60);
                        plan.addActivity(activity2);
                        plan.addLeg(populationFactory.createLeg(mode.get(m)));

                    } else {
                        //if the person is old student
                        start_time = 7 * 60 * 60 + random.nextInt(7200); //start time of first activity
                        Activity activity1 = populationFactory.createActivityFromCoord("home", transformation.transform(new Coord(y.get(home - 1) + blur_loc, x.get(home - 1) + blur_loc)));
                        activity1.setEndTime(start_time - (random.nextInt(30 * 60) + 30 * 60));
                        plan.addActivity(activity1);
                        plan.addLeg(populationFactory.createLeg(mode.get(m)));

                        Activity activity2 = populationFactory.createActivityFromCoord("education", transformation.transform(new Coord(y.get(random.nextInt(x.size()) ) + blur_loc, x.get(random.nextInt(y.size()) ) + blur_loc)));
                        end_time = start_time + random.nextInt(4 * 60 * 60) + 6 * 60 * 60;
                        activity2.setStartTime(start_time); // start time from 07.00 to 09.00
                        activity2.setEndTime(end_time);
                        plan.addActivity(activity2);
                        plan.addLeg(populationFactory.createLeg(mode.get(m)));


                        // old students have leisure activities before midnight


                        Activity act_other = populationFactory.createActivityFromCoord(activity_second.get(random.nextInt(3)), transformation.transform(new Coord(y.get(random.nextInt(x.size()) ) + blur_loc, x.get(random.nextInt(y.size()) ) + blur_loc)));
                        start_time = end_time + 60 * 60;
                        end_time = start_time + random.nextInt(4 * 60 * 60);
                        act_other.setStartTime(start_time);
                        act_other.setEndTime(end_time);
                        plan.addActivity(act_other);
                        plan.addLeg(populationFactory.createLeg(mode.get(m)));

                    }
                } else if (rdE == 1) {
                    //System.out.println(reg_worker);
                    reg_worker = reg_worker + 1;
                    // regular workers, first activity is work, then leisure/shopping
                    start_time = 6 * 60 * 60 + random.nextInt(3 * 60 * 60);
                    Activity activity1 = populationFactory.createActivityFromCoord("home", transformation.transform(new Coord(y.get(home - 1) + blur_loc, x.get(home - 1) + blur_loc)));
                    activity1.setEndTime(start_time - (random.nextInt(40 * 60) + 40));
                    plan.addActivity(activity1);
                    plan.addLeg(populationFactory.createLeg(mode.get(m)));
                    end_time = start_time + random.nextInt(3 * 60 * 60) + 6 * 60 * 60;
                    Activity activity2 = populationFactory.createActivityFromCoord("work", transformation.transform(new Coord(y.get(random.nextInt(x.size()) ) + blur_loc, x.get(random.nextInt(y.size())) + blur_loc)));
                    activity2.setStartTime(start_time);
                    activity2.setEndTime(end_time);
                    plan.addActivity(activity2);
                    plan.addLeg(populationFactory.createLeg(mode.get(m)));


                    //after work can have two other activities (shopping, leisure, other)
                    for (int i = 0; i < random.nextInt(2) + 1; i++) {
                        start_time = end_time + random.nextInt(1 * 60 * 60) + 30 * 60;
                        end_time = start_time + random.nextInt(1 * 60 * 60);
                        Activity act_other = populationFactory.createActivityFromCoord(activity_second.get(random.nextInt(3)), transformation.transform(new Coord(y.get(random.nextInt(x.size()) ) + blur_loc, x.get(random.nextInt(y.size()) ) + blur_loc)));
                        act_other.setStartTime(start_time);
                        act_other.setEndTime(end_time);
                        plan.addActivity(act_other);
                        plan.addLeg(populationFactory.createLeg(mode.get(m)));
                    }


                } else {
                    //retired, can have up to 3 activities
                    start_time = 9 * 60 * 60 + random.nextInt(2 * 60 * 60);
                    Activity activity1 = populationFactory.createActivityFromCoord("home", transformation.transform(new Coord(y.get(home - 1) + blur_loc, x.get(home - 1) + blur_loc)));
                    activity1.setEndTime(start_time - random.nextInt(40 * 60));
                    plan.addActivity(activity1);
                    plan.addLeg(populationFactory.createLeg(mode.get(m)));
                    for (int i = 0; i < random.nextInt(3) + 1; i++) {
                        end_time = start_time + random.nextInt(40 * 60) + 40 * 60;
                        Activity act_other = populationFactory.createActivityFromCoord(activity_second.get(random.nextInt(3)), transformation.transform(new Coord(y.get(random.nextInt(x.size()) ) + blur_loc, x.get(random.nextInt(y.size()) ) + blur_loc)));
                        act_other.setStartTime(start_time);
                        act_other.setEndTime(end_time);
                        plan.addActivity(act_other);
                        plan.addLeg(populationFactory.createLeg(mode.get(m)));
                        start_time = end_time + random.nextInt(2 * 60 * 60) + 30 * 60;
                    }
                }
                Activity activity3 = populationFactory.createActivityFromCoord("home", transformation.transform(new Coord(y.get(home - 1) + blur_loc, x.get(home - 1) + blur_loc)));
                plan.addActivity(activity3);
                person.addPlan(plan);
            }

        }

//        System.out.println("DONE");

        MatsimWriter popWriter = new PopulationWriter(population, network); //We finally add the person to file
        popWriter.write(Paths.get("data", "amsterdam", "original_input", "networks", net_id, "plans", plan_id, "plans.xml").toString());
        microdata.close();

    }

    public static void main(String[] args) throws IOException {
//        generateTravelDemand("2","2",1,true);
    }
}
