package si.ijs.urbanite.simserver.utils;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import java.util.HashMap;

public class EmissionHandler extends DefaultHandler {
    public static HashMap<Integer, HashMap<String, Double>> emissionMap = new HashMap<Integer,  HashMap<String, Double>>();

    @Override
    public void startElement(String uri, String localName, String qName, Attributes attributes) throws SAXException {

        if (qName.equalsIgnoreCase("event")) {

            String linkIdStr = attributes.getValue("linkId");
            String type = attributes.getValue("type");

            Double co = Double.parseDouble(attributes.getValue("CO"));
            Double co2 = Double.parseDouble(attributes.getValue("CO2_TOTAL"));
            Double fc = Double.parseDouble(attributes.getValue("FC"));
            Double nox = Double.parseDouble(attributes.getValue("NOx"));
            Double no2 = Double.parseDouble(attributes.getValue("NO2"));

            if (type.equals("warmEmissionEvent") || type.equals("coldEmissionEvent")) { // ignore other event types

                int linkId;
                if(Character.isDigit(linkIdStr.charAt(0))) { // normal link id is numerical
                    linkId = Integer.parseInt(linkIdStr);
                }
                else if(linkIdStr.charAt(0) == 'p' && linkIdStr.length() < 9) { // virtual PT link is "pt_<linkId>"
                    String subLink = linkIdStr.substring(3, linkIdStr.length());
                    linkId = Integer.parseInt(subLink);
                }
                else {
                    return; // ignore non-links
                }

                if (!emissionMap.containsKey(linkId)) {
                    emissionMap.put(linkId, new HashMap<String, Double>());
                    emissionMap.get(linkId).put("CO", co);
                    emissionMap.get(linkId).put("CO2", co2);
                    emissionMap.get(linkId).put("FC", fc);
                    emissionMap.get(linkId).put("NOX", nox);
                    emissionMap.get(linkId).put("NO2", no2);
                }
                else {
                    if (emissionMap.get(linkId).containsKey("CO")) {
                        emissionMap.get(linkId).put("CO", emissionMap.get(linkId).get("CO") + co);
                    }
                    if (emissionMap.get(linkId).containsKey("CO2")) {
                        emissionMap.get(linkId).put("CO2", emissionMap.get(linkId).get("CO2") + co2);
                    }
                    if (emissionMap.get(linkId).containsKey("FC")) {
                        emissionMap.get(linkId).put("FC", emissionMap.get(linkId).get("FC") + fc);
                    }
                    if (emissionMap.get(linkId).containsKey("NOX")) {
                        emissionMap.get(linkId).put("NOX", emissionMap.get(linkId).get("NOX") + nox);
                    }
                    if (emissionMap.get(linkId).containsKey("NO2")) {
                        emissionMap.get(linkId).put("NO2", emissionMap.get(linkId).get("NO2") + no2);
                    }
                }
            }
        }
    }
}
