package si.ijs.urbanite.simserver.utils;

import org.json.simple.JSONObject;
import org.json.simple.parser.JSONParser;
import org.json.simple.parser.ParseException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.ResponseEntity;
import org.springframework.web.client.RestTemplate;
import si.ijs.urbanite.simserver.simulation.SimulationRunService;

import java.io.*;
import java.net.URL;
import java.net.URLConnection;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.StandardCopyOption;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

public class HelperFunctions {
    private final static Logger logger = LoggerFactory.getLogger(SimulationRunService.class);
    public static List<Path> findFilesThatContain(Path path, String[] fileExtensions) throws IOException {
        List<Path> result;
        try (Stream<Path> walk = Files.walk(path, 1)) {
            result = walk
                    .filter(p -> !Files.isDirectory(p))
                    // convert path to string
                    .filter(f -> contains(f.toString().toLowerCase(), fileExtensions))
                    .collect(Collectors.toList());
        }
        return result;
    }

    public static List<Path> findFilesThatMatches(Path path, String[] fileExtensions) throws IOException {
        List<Path> result;
        try (Stream<Path> walk = Files.walk(path, 1)) {
            result = walk
                    .filter(p -> !Files.isDirectory(p))
                    // convert path to string
                    .filter(f -> matches(f.toString().toLowerCase(), fileExtensions))
                    .collect(Collectors.toList());
        }
        return result;
    }
    private static boolean contains(String file, String[] fileExtensions) {
        //we are doing new File(file).getName(), because the input "String file" parameter is an absolute path
        boolean result = Arrays.stream(fileExtensions).anyMatch(new File(file).getName()::contains);
        return result;
    }
    private static boolean matches(String file, String[] fileExtensions) {
        //we are doing new File(file).getName(), because the input "String file" parameter is an absolute path
        boolean result = Arrays.stream(fileExtensions).anyMatch(new File(file).getName()::equals);
        return result;
    }

    public static void copyFile(InputStream source, OutputStream target) throws IOException {
        byte[] buf = new byte[8192];
        int length;
        while ((length = source.read(buf)) > 0) {
            target.write(buf, 0, length);
        }
    }

    public static void copyFile(String source_path, String destination) throws IOException {
        Files.copy(Paths.get(source_path), Paths.get(destination), StandardCopyOption.REPLACE_EXISTING);
    }

    public static void logExternalProcess(Logger logger, BufferedReader bfr, String name) throws IOException {
        boolean showHeader = true;
        String line;
        while ((line = bfr.readLine()) != null){
            if (showHeader) {
                logger.info(String.format("Error stream for %s", name));
                showHeader = false;
            }
            logger.error(line);
        }
        logger.info(String.format("Process finished %s", name));
    }

}
