package si.ijs.urbanite.simserver.utils;

import org.geotools.geometry.jts.JTS;
import org.geotools.referencing.CRS;
import org.jfree.data.json.impl.JSONArray;
import org.jfree.data.json.impl.JSONObject;

import org.locationtech.jts.geom.Coordinate;
import org.locationtech.jts.geom.CoordinateXY;
import org.opengis.referencing.FactoryException;
import org.opengis.referencing.crs.CoordinateReferenceSystem;
import org.opengis.referencing.operation.MathTransform;
import org.opengis.referencing.operation.TransformException;

import javax.validation.constraints.NotNull;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;


public class ProcessResults {

    // counts how many bicycles and cars has entered the Moyua square
    public static HashMap<String, ArrayList<Integer>> BILcountVehIntoMoyua(File eventsFile, ArrayList<String> links) throws IOException {
        FileReader fr = new FileReader(eventsFile);
        BufferedReader br = new BufferedReader(fr);
        String line;

        boolean allLinks = false;
        if (links == null) {
            allLinks = true;
        }

        // hand implemented XML read - TODO replace with library implementation if needed
        HashMap<String, ArrayList<String>> linkStats = new HashMap<String, ArrayList<String>>();
        HashMap<String, ArrayList<Integer>> linkStatsFinal = new HashMap<>();

        while ((line = br.readLine()) != null) {
            //time is 05:50 am - 06.10 am (21 000 sec - 22 200 sec)
            if (line.matches("(.*)time(.*)")) {
                double time = Double.parseDouble(line.split("time=\"")[1].split("\"")[0]);
                // TODO: parametrize time constraints
                if (time >= 21000.0 && time <= 22200.0) {
                    String eventType = line.split("type=\"")[1].split("\"")[0];
                    if (eventType.matches("entered link")) {
                        String linkId = line.split("link=\"")[1].split("\"")[0];
                        if (allLinks || links.contains(linkId)) {
//                            System.out.println(line);
                            ArrayList<String> vehicles = new ArrayList<String>();
                            if (linkStats.containsKey(linkId)) {
                                vehicles = linkStats.get(linkId);
                                vehicles.add(line.split("vehicle=\"")[1].split("\"")[0]);
                                linkStats.put(linkId, vehicles);
                            } else {
                                vehicles.add(line.split("vehicle=\"")[1].split("\"")[0]);
                                linkStats.put(linkId, vehicles);
                            }
                        }
                    }
                }
            }
        }
        System.out.println(linkStats);
        System.out.println("link id:\tnum of cars\tnum of bicycles\tnum of pt");
        for (String linkId : linkStats.keySet()) {
            ArrayList<String> vehicles = linkStats.get(linkId);
            Integer cars = 0;
            Integer bicycles = 0;
            Integer buses = 0;
            for (String v : vehicles) {
                // TODO parametrize vehicle type matching
                if (v.matches("(.*)bicycle(.*)")) {
                    bicycles++;
                } else if (v.matches("(.*)pt(.*)")) {
                    buses++;
                } else {
                    cars++;
                }
            }
            ArrayList<Integer> stats = new ArrayList<>();
            stats.add(cars);
            stats.add(bicycles);
            stats.add(buses);
            linkStatsFinal.put(linkId, stats);
            System.out.println(linkId + "\t" + cars + "\t" + bicycles + "\t" + buses);
        }
        return linkStatsFinal;
    }

    public static JSONObject link2geo(ArrayList<String> links, File networkFile, File eventsFile) throws IOException {
        SaxParser.parseXML(networkFile);
        SaxParser.parseXML(eventsFile);

        HashMap<Integer, Integer> lookupEvents = EventHandler.map;
        HashMap<String, ArrayList<Double>> coordinatesNetwork = NetworkHandler.coordinatesNetwork;
        HashMap<String, ArrayList<String>> linksNetwork = NetworkHandler.linksNetwork;

        JSONObject featureCollection = new JSONObject();
        featureCollection.put("type", "FeatureCollection");

        JSONArray features = new JSONArray();
        if (!links.isEmpty()) {
            for (String link : links) {
                fillFeatureCollection(link, features, featureCollection, coordinatesNetwork, linksNetwork, lookupEvents);
            }
        } else {
            for (String link : linksNetwork.keySet()) {
                fillFeatureCollection(link, features, featureCollection, coordinatesNetwork, linksNetwork, lookupEvents);
            }
        }
        String str = featureCollection.toString();
        ;
        String fileName = "C:\\Users\\ziga\\data\\test.txt";
        BufferedWriter writer = new BufferedWriter(new FileWriter(fileName, true));
        writer.append(str);
        System.out.println(featureCollection);
        writer.close();

        return featureCollection;
    }

    // write JSON file (count entry vehicles merged into JSON)
    public static JSONArray BILcountVehIntoMoyuaJSON(HashMap<String, ArrayList<Integer>> linkStats) {
        JSONObject obj = new JSONObject();
        JSONArray results = new JSONArray();

        // TODO replace hardcoded with iteration over all links in network (when linkStats null)
        // TODO replace hardcoded with iteration over provided links in network (when linkStats not null)

        obj.put("link id", 15557);
        obj.put("end_1", new CoordinateXY(43.2626162, -2.9331623));
        obj.put("end_2", new CoordinateXY(43.2628227, -2.9340971));
        obj.put("cars", linkStats.get(String.valueOf(15557)).get(0));
        obj.put("bikes", linkStats.get(String.valueOf(15557)).get(1));
        obj.put("buses", linkStats.get(String.valueOf(15557)).get(2));
        results.add(obj);

        obj = new JSONObject();
        obj.put("link id", 15558);
        obj.put("end_1", new CoordinateXY(43.2626162, -2.9331623));
        obj.put("end_2", new CoordinateXY(43.2628227, -2.9340971));
        obj.put("cars", linkStats.get(String.valueOf(15558)).get(0));
        obj.put("bikes", linkStats.get(String.valueOf(15558)).get(1));
        obj.put("buses", linkStats.get(String.valueOf(15558)).get(2));
        results.add(obj);

        obj = new JSONObject();
        obj.put("link id", 10405);
        obj.put("end_1", new CoordinateXY(43.2620120, -2.9353727));
        obj.put("end_2", new CoordinateXY(43.2625225, -2.9351731));
        obj.put("cars", linkStats.get(String.valueOf(10405)).get(0));
        obj.put("bikes", linkStats.get(String.valueOf(10405)).get(1));
        obj.put("buses", linkStats.get(String.valueOf(10405)).get(2));
        results.add(obj);

        obj = new JSONObject();
        obj.put("link id", 10406);
        obj.put("end_1", new CoordinateXY(43.2620120, -2.9353727));
        obj.put("end_2", new CoordinateXY(43.2625225, -2.9351731));
        obj.put("cars", linkStats.get(String.valueOf(10406)));
        obj.put("bikes", linkStats.get(String.valueOf(10406)));
        obj.put("buses", linkStats.get(String.valueOf(10406)));
        results.add(obj);

        obj = new JSONObject();
        obj.put("link id", 15621);
        obj.put("end_1", new CoordinateXY(43.2622188, -2.9366817));
        obj.put("end_2", new CoordinateXY(43.2626760, -2.9357903));
        obj.put("cars", linkStats.get(String.valueOf(15621)).get(0));
        obj.put("bikes", linkStats.get(String.valueOf(15621)).get(1));
        obj.put("buses", linkStats.get(String.valueOf(15621)).get(2));
        results.add(obj);

        obj = new JSONObject();
        obj.put("link id", 15622);
        obj.put("end_1", new CoordinateXY(43.2622188, -2.9366817));
        obj.put("end_2", new CoordinateXY(43.2626760, -2.9357903));
        obj.put("cars", linkStats.get(String.valueOf(15622)).get(0));
        obj.put("bikes", linkStats.get(String.valueOf(15622)).get(1));
        obj.put("buses", linkStats.get(String.valueOf(15622)).get(2));
        results.add(obj);

        obj = new JSONObject();
        obj.put("link id", 8067);
        obj.put("end_1", new CoordinateXY(43.2632753, -2.9361892));
        obj.put("end_2", new CoordinateXY(43.2631574, -2.9356640));
        obj.put("cars", linkStats.get(String.valueOf(8067)).get(0));
        obj.put("bikes", linkStats.get(String.valueOf(8067)).get(1));
        obj.put("buses", linkStats.get(String.valueOf(8067)).get(2));
        results.add(obj);

        obj = new JSONObject();
        obj.put("link id", 8068);
        obj.put("end_1", new CoordinateXY(43.2632753, -2.9361892));
        obj.put("end_2", new CoordinateXY(43.2631574, -2.9356640));
        obj.put("cars", linkStats.get(String.valueOf(8068)).get(0));
        obj.put("bikes", linkStats.get(String.valueOf(8068)).get(1));
        obj.put("buses", linkStats.get(String.valueOf(8068)).get(2));
        results.add(obj);

        obj = new JSONObject();
        obj.put("link id", 5958);
        obj.put("end_1", new CoordinateXY(43.2638867, -2.9346258));
        obj.put("end_2", new CoordinateXY(43.2634626, -2.9348000));
        obj.put("cars", linkStats.get(String.valueOf(5958)).get(0));
        obj.put("bikes", linkStats.get(String.valueOf(5958)).get(1));
        obj.put("buses", linkStats.get(String.valueOf(5958)).get(2));
        results.add(obj);

        obj = new JSONObject();
        obj.put("link id", 5957);
        obj.put("end_1", new CoordinateXY(43.2638867, -2.9346258));
        obj.put("end_2", new CoordinateXY(43.2634626, -2.9348000));
        obj.put("cars", linkStats.get(String.valueOf(5957)).get(0));
        obj.put("bikes", linkStats.get(String.valueOf(5957)).get(1));
        obj.put("buses", linkStats.get(String.valueOf(5957)).get(2));
        results.add(obj);

        obj = new JSONObject();
        obj.put("link id", 733);
        obj.put("end_1", new CoordinateXY(43.2636769, -2.9335347));
        obj.put("end_2", new CoordinateXY(43.2632716, -2.9343442));
        obj.put("cars", linkStats.get(String.valueOf(733)).get(0));
        obj.put("bikes", linkStats.get(String.valueOf(733)).get(1));
        obj.put("buses", linkStats.get(String.valueOf(733)).get(2));
        results.add(obj);

        obj = new JSONObject();
        obj.put("link id", 14560);
        obj.put("end_1", new CoordinateXY(43.2618132, -2.9343165));
        obj.put("end_2", new CoordinateXY(43.2625004, -2.9346978));
        obj.put("cars", linkStats.get(String.valueOf(14560)).get(0));
        obj.put("bikes", linkStats.get(String.valueOf(14560)).get(1));
        obj.put("buses", linkStats.get(String.valueOf(14560)).get(2));
        results.add(obj);

        return results;
    }

    public static Collection<String> stringToCollection(@NotNull String s) {
        String[] strings = s.split(",");
        Collection<String> stringsClean = new ArrayList<>();
        for (String string : strings)
            stringsClean.add(string.trim());
        return stringsClean;
    }

    public static void fillFeatureCollection(String link, JSONArray features, JSONObject featureCollection, HashMap<String, ArrayList<Double>> coordinatesNetwork, HashMap<String, ArrayList<String>> linksNetwork, HashMap<Integer, Integer> lookupEvents) {

        //System.out.println("Link: " + link);
        if (linksNetwork.containsKey(link)) {
            //System.out.println("From: " + linksNetwork.get(link).get(0) + " to:" + linksNetwork.get(link).get(1));
            double firstX = 0;
            double firstY = 0;
            double secondX = 0;
            double secondY = 0;

            if (coordinatesNetwork.containsKey(linksNetwork.get(link).get(0))) {
                firstX = coordinatesNetwork.get(linksNetwork.get(link).get(0)).get(0);
                firstY = coordinatesNetwork.get(linksNetwork.get(link).get(0)).get(1);
                //System.out.println(firstX + " " + firstY);
            }
            if (coordinatesNetwork.containsKey(linksNetwork.get(link).get(1))) {
                secondX = coordinatesNetwork.get(linksNetwork.get(link).get(1)).get(0);
                secondY = coordinatesNetwork.get(linksNetwork.get(link).get(1)).get(1);
                //System.out.println(secondX + " " + secondY);
            }

            Integer parsedLink = Integer.parseInt(link);

            if (lookupEvents.containsKey(parsedLink)) {
                //System.out.println("Events: " + lookupEvents.get(parsedLink));

                JSONObject feature = new JSONObject();
                feature.put("type", "Feature");
                feature.put("link", link);
                feature.put("carCount", lookupEvents.get(parsedLink));
                JSONObject geometry = new JSONObject();

                JSONArray JSONArrayCoord = new JSONArray();
                JSONArray firstJsonArray = new JSONArray();

                JSONArray secondJSONCoord = new JSONArray();
                JSONArrayCoord.add(0, firstX);
                JSONArrayCoord.add(1, firstY);
                secondJSONCoord.add(0, secondX);
                secondJSONCoord.add(1, secondY);

                firstJsonArray.add(JSONArrayCoord);
                firstJsonArray.add(secondJSONCoord);
                geometry.put("type", "LineString");
                geometry.put("coordinates", firstJsonArray);
                feature.put("geometry", geometry);
                features.add(feature);

                featureCollection.put("features", features);
            }
        }
    }

    public static void getEmissionCount(ArrayList<String> links, File emissionFile) {
        SaxParser.parseXML(emissionFile);
        HashMap<Integer, HashMap<String, Double>> emissionMap = EmissionHandler.emissionMap;

        JSONObject featureCollection = new JSONObject();
        featureCollection.put("type", "FeatureCollection");
        JSONArray features = new JSONArray();

        if (links.isEmpty()) {
            for (Integer link : emissionMap.keySet()) {
                JSONObject feature = new JSONObject();
                feature.put("type", "Feature");
                feature.put("link", link);
                feature.put("CO", emissionMap.get(link).get("CO"));
                feature.put("CO2", emissionMap.get(link).get("CO2"));
                feature.put("FC", emissionMap.get(link).get("FC"));
                feature.put("NO2", emissionMap.get(link).get("NO2"));
                feature.put("NOX", emissionMap.get(link).get("NOX"));
                features.add(feature);
                featureCollection.put("features", features);
            }
            System.out.println(featureCollection);
        } else {
            for (String link : links) {
                int intLink = Integer.parseInt(link);

                JSONObject feature = new JSONObject();
                feature.put("type", "Feature");
                feature.put("link", intLink);
                feature.put("CO", emissionMap.get(intLink).get("CO"));
                feature.put("CO2", emissionMap.get(intLink).get("CO2"));
                feature.put("FC", emissionMap.get(intLink).get("FC"));
                feature.put("NO2", emissionMap.get(intLink).get("NO2"));
                feature.put("NOX", emissionMap.get(intLink).get("NOX"));
                features.add(feature);
                featureCollection.put("features", features);
            }
            System.out.println(featureCollection);
        }
    }

    public static void returnEmissionCountGeoJson(ArrayList<String> links, File emissionFile, File networkFile, File eventsFile) throws IOException, FactoryException, TransformException {
        SaxParser.parseXML(emissionFile);
        SaxParser.parseXML(networkFile);
        SaxParser.parseXML(eventsFile);

        HashMap<Integer, Integer> lookupEvents = EventHandler.map;
        HashMap<String, ArrayList<Double>> coordinatesNetwork = NetworkHandler.coordinatesNetwork;
        HashMap<String, ArrayList<String>> linksNetwork = NetworkHandler.linksNetwork;
        HashMap<Integer, HashMap<String, Double>> emissionMap = EmissionHandler.emissionMap;

        JSONObject featureCollection = new JSONObject();
        featureCollection.put("type", "FeatureCollection");
        JSONArray features = new JSONArray();

        if (links.isEmpty()) {
            for (Integer link : emissionMap.keySet()) {
                JSONObject feature = new JSONObject();
                JSONObject properties = new JSONObject();
                feature.put("type", "Feature");
                properties.put("link", link);
                properties.put("CO", emissionMap.get(link).get("CO"));
                properties.put("CO2", emissionMap.get(link).get("CO2"));
                properties.put("FC", emissionMap.get(link).get("FC"));
                properties.put("NO2", emissionMap.get(link).get("NO2"));
                properties.put("NOX", emissionMap.get(link).get("NOX"));

                String newLink = Integer.toString(link);
                if (linksNetwork.containsKey(newLink)) {
                    double firstX = 0;
                    double firstY = 0;
                    double secondX = 0;
                    double secondY = 0;

                    if (coordinatesNetwork.containsKey(linksNetwork.get(newLink).get(0))) {
                        firstX = coordinatesNetwork.get(linksNetwork.get(newLink).get(0)).get(0);
                        firstY = coordinatesNetwork.get(linksNetwork.get(newLink).get(0)).get(1);

                        CoordinateReferenceSystem sourceCRS = CRS.decode("EPSG:2062");
                        CoordinateReferenceSystem targetCRS = CRS.decode("EPSG:4326");
                        MathTransform transform = CRS.findMathTransform(sourceCRS, targetCRS, true);
                        Coordinate in = new Coordinate(firstX, firstY);
                        Coordinate out = in;
                        Coordinate result = JTS.transform(in, out, transform);

                        firstX = result.x;
                        firstY = result.y;
                    }

                    if (coordinatesNetwork.containsKey(linksNetwork.get(newLink).get(1))) {
                        secondX = coordinatesNetwork.get(linksNetwork.get(newLink).get(1)).get(0);
                        secondY = coordinatesNetwork.get(linksNetwork.get(newLink).get(1)).get(1);

                        CoordinateReferenceSystem sourceCRS = CRS.decode("EPSG:2062");
                        CoordinateReferenceSystem targetCRS = CRS.decode("EPSG:4326");
                        MathTransform transform = CRS.findMathTransform(sourceCRS, targetCRS, true);
                        Coordinate in = new Coordinate(secondX, secondY);
                        Coordinate out = in;
                        Coordinate result = JTS.transform(in, out, transform);
                        secondX = result.x;
                        secondY = result.y;
                    }

                    Integer parsedLink = Integer.parseInt(newLink);

                    if (lookupEvents.containsKey(parsedLink)) {
                        properties.put("carCount", lookupEvents.get(parsedLink));
                        feature.put("properties", properties);
                        JSONObject geometry = new JSONObject();

                        JSONArray JSONArrayCoord = new JSONArray();
                        JSONArray firstJsonArray = new JSONArray();

                        JSONArray secondJSONCoord = new JSONArray();
                        JSONArrayCoord.add(0, firstX);
                        JSONArrayCoord.add(1, firstY);
                        secondJSONCoord.add(0, secondX);
                        secondJSONCoord.add(1, secondY);

                        firstJsonArray.add(JSONArrayCoord);
                        firstJsonArray.add(secondJSONCoord);
                        geometry.put("type", "LineString");
                        geometry.put("coordinates", firstJsonArray);
                        feature.put("geometry", geometry);
                    }
                }
                features.add(feature);
                featureCollection.put("features", features);
            }
            System.out.println(featureCollection);
            String str = featureCollection.toString();
            ;
            String fileName = "C:\\Users\\ziga\\data\\test.txt";
            BufferedWriter writer = new BufferedWriter(new FileWriter(fileName, true));
            writer.append(str);
        } else {
            for (String link : links) {
                int intLink = Integer.parseInt(link);
                JSONObject feature = new JSONObject();
                JSONObject properties = new JSONObject();
                feature.put("type", "Feature");
                properties.put("link", intLink);
                properties.put("CO", emissionMap.get(intLink).get("CO"));
                properties.put("CO2", emissionMap.get(intLink).get("CO2"));
                properties.put("FC", emissionMap.get(intLink).get("FC"));
                properties.put("NO2", emissionMap.get(intLink).get("NO2"));
                properties.put("NOX", emissionMap.get(intLink).get("NOX"));

                String newLink = link;
                if (linksNetwork.containsKey(newLink)) {
                    double firstX = 0;
                    double firstY = 0;
                    double secondX = 0;
                    double secondY = 0;

                    if (coordinatesNetwork.containsKey(linksNetwork.get(newLink).get(0))) {
                        firstX = coordinatesNetwork.get(linksNetwork.get(newLink).get(0)).get(0);
                        firstY = coordinatesNetwork.get(linksNetwork.get(newLink).get(0)).get(1);
                        System.out.println(firstX + " " + firstY);

                        CoordinateReferenceSystem sourceCRS = CRS.decode("EPSG:2062");
                        CoordinateReferenceSystem targetCRS = CRS.decode("EPSG:4326");
                        MathTransform transform = CRS.findMathTransform(sourceCRS, targetCRS, true);
                        Coordinate in = new Coordinate(firstX, firstY);
                        Coordinate out = in;
                        Coordinate result = JTS.transform(in, out, transform);

                        firstX = result.x;
                        firstY = result.y;

                    }
                    if (coordinatesNetwork.containsKey(linksNetwork.get(newLink).get(1))) {
                        secondX = coordinatesNetwork.get(linksNetwork.get(newLink).get(1)).get(0);
                        secondY = coordinatesNetwork.get(linksNetwork.get(newLink).get(1)).get(1);

                        CoordinateReferenceSystem sourceCRS = CRS.decode("EPSG:2062");
                        CoordinateReferenceSystem targetCRS = CRS.decode("EPSG:4326");
                        MathTransform transform = CRS.findMathTransform(sourceCRS, targetCRS, true);
                        Coordinate in = new Coordinate(secondX, secondY);
                        Coordinate out = in;
                        Coordinate result = JTS.transform(in, out, transform);

                        secondX = result.x;
                        secondY = result.y;
                    }

                    Integer parsedLink = Integer.parseInt(newLink);

                    if (lookupEvents.containsKey(parsedLink)) {
                        properties.put("carCount", lookupEvents.get(parsedLink));
                        feature.put("properties", properties);
                        JSONObject geometry = new JSONObject();

                        JSONArray JSONArrayCoord = new JSONArray();
                        JSONArray firstJsonArray = new JSONArray();

                        JSONArray secondJSONCoord = new JSONArray();
                        JSONArrayCoord.add(0, firstX);
                        JSONArrayCoord.add(1, firstY);
                        secondJSONCoord.add(0, secondX);
                        secondJSONCoord.add(1, secondY);

                        firstJsonArray.add(JSONArrayCoord);
                        firstJsonArray.add(secondJSONCoord);
                        geometry.put("type", "LineString");
                        geometry.put("coordinates", firstJsonArray);
                        feature.put("geometry", geometry);
                    }
                }
                features.add(feature);
                featureCollection.put("features", features);
            }
            System.out.println(featureCollection);
        }
    }

    public static void returnTimeGeoJson(File amsterdamFile) throws IOException {
        SaxParser.parseXML(amsterdamFile);

        JSONObject featureCollection = new JSONObject();
        featureCollection.put("type", "FeatureCollection");
        JSONArray features = new JSONArray();

        for (int i : AmsterdamHandler.timeStamp.keySet()) {
            ArrayList<Integer> listLinks = AmsterdamHandler.timeStamp.get(i);
            for (int link : listLinks) {
                JSONObject feature = new JSONObject();
                feature.put("type", "Feature");
                feature.put("link", link);
                feature.put("time", i);
                feature.put("linkCount", AmsterdamHandler.amsterdamMap.get(link));
                features.add(feature);
                featureCollection.put("features", features);
            }
        }
        String str = featureCollection.toString();
        ;
        String fileName = "C:\\Users\\ziga\\data\\bilbaoTimeScenarioOutput.txt";
        BufferedWriter writer = new BufferedWriter(new FileWriter(fileName, true));
        writer.append(str);
        writer.close();
    }

    public static void returnTimeGeoJsonGeneral(File amsterdamFile, File networkFile) throws IOException, FactoryException, TransformException {
        SaxParser.parseXML(amsterdamFile);
        SaxParser.parseXML(networkFile);

        HashMap<String, ArrayList<Double>> coordinatesNetwork = NetworkHandler.coordinatesNetwork;
        HashMap<String, ArrayList<String>> linksNetwork = NetworkHandler.linksNetwork;

        JSONObject featureCollection = new JSONObject();
        featureCollection.put("type", "FeatureCollection");
        JSONArray features = new JSONArray();


        for (int i : AmsterdamHandler.timeStamp.keySet()) {
            ArrayList<Integer> listLinks = AmsterdamHandler.timeStamp.get(i);
            //System.out.println("Time: " + i);
            for (int link : listLinks) {
                JSONObject feature = new JSONObject();
                JSONObject properties = new JSONObject();
                feature.put("type", "Feature");
                properties.put("link", link);
                properties.put("time", i);
                properties.put("linkCount", AmsterdamHandler.amsterdamMap.get(link));
                feature.put("properties", properties);
                String newLink = Integer.toString(link);
                if (linksNetwork.containsKey(newLink)) {
                    //System.out.println("From: " + linksNetwork.get(link).get(0) + " to:" + linksNetwork.get(link).get(1));
                    double firstX = 0;
                    double firstY = 0;
                    double secondX = 0;
                    double secondY = 0;

                    if (coordinatesNetwork.containsKey(linksNetwork.get(newLink).get(0))) {
                        firstX = coordinatesNetwork.get(linksNetwork.get(newLink).get(0)).get(0);
                        firstY = coordinatesNetwork.get(linksNetwork.get(newLink).get(0)).get(1);

                        CoordinateReferenceSystem sourceCRS = CRS.decode("EPSG:28992");
                        CoordinateReferenceSystem targetCRS = CRS.decode("EPSG:4326");
                        MathTransform transform = CRS.findMathTransform(sourceCRS, targetCRS, true);
                        Coordinate in = new Coordinate(firstX, firstY);
                        Coordinate out = in;
                        Coordinate result = JTS.transform(in, out, transform);

                        //System.out.println(result.toString());
                        //System.out.println(result.x + " " + result.y);
                        firstX = result.x;
                        firstY = result.y;
                        //System.out.println(firstX + " " + firstY);
                    }
                    if (coordinatesNetwork.containsKey(linksNetwork.get(newLink).get(1))) {
                        secondX = coordinatesNetwork.get(linksNetwork.get(newLink).get(1)).get(0);
                        secondY = coordinatesNetwork.get(linksNetwork.get(newLink).get(1)).get(1);
                        //System.out.println(secondX + " " + secondY);

                        CoordinateReferenceSystem sourceCRS = CRS.decode("EPSG:28992");
                        CoordinateReferenceSystem targetCRS = CRS.decode("EPSG:4326");
                        MathTransform transform = CRS.findMathTransform(sourceCRS, targetCRS, true);
                        Coordinate in = new Coordinate(secondX, secondY);
                        Coordinate out = in;
                        Coordinate result = JTS.transform(in, out, transform);

                        //System.out.println(result.toString());
                        //System.out.println(result.x + " " + result.y);
                        secondX = result.x;
                        secondY = result.y;
                    }

                    JSONObject geometry = new JSONObject();

                    JSONArray JSONArrayCoord = new JSONArray();
                    JSONArray firstJsonArray = new JSONArray();

                    JSONArray secondJSONCoord = new JSONArray();
                    JSONArrayCoord.add(0, firstX);
                    JSONArrayCoord.add(1, firstY);
                    secondJSONCoord.add(0, secondX);
                    secondJSONCoord.add(1, secondY);

                    firstJsonArray.add(JSONArrayCoord);
                    firstJsonArray.add(secondJSONCoord);
                    geometry.put("type", "LineString");
                    geometry.put("coordinates", firstJsonArray);
                    feature.put("geometry", geometry);

                }
                features.add(feature);
                featureCollection.put("features", features);

            }
        }
        System.out.println(featureCollection);
        String str = featureCollection.toString();
        ;
        String fileName = "C:\\Users\\ziga\\data\\test.txt";
        BufferedWriter writer = new BufferedWriter(new FileWriter(fileName, true));
        writer.append(str);

        writer.close();

    }

}
