package si.ijs.urbanite.simserver.vehicles;

import org.matsim.api.core.v01.Id;
import org.matsim.api.core.v01.Scenario;
import org.matsim.api.core.v01.network.Link;
import org.matsim.core.config.Config;
import org.matsim.core.config.ConfigUtils;
import org.matsim.core.controler.Controler;
import org.matsim.core.controler.OutputDirectoryHierarchy;
import org.matsim.core.scenario.ScenarioUtils;
import org.matsim.vehicles.VehicleType;
import org.matsim.vehicles.VehicleUtils;
import org.matsim.vehicles.VehicleWriterV1;

import java.nio.file.Paths;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;

public class VehiclesGeneration {
    public static Controler addBicycleTypes(Config config){
        Scenario sc = ScenarioUtils.loadScenario(config);

/* This is commented for Amsterdam because when running the pipeline for a second time/or if a vehicles.xml files already
exists in simulations folder, then addVehicleType() produces an error because the bicycle types(slow,fast,medium) are already defined.
The vehicles.xml file with already defined bicycle types is attached on onedrive.

*  */
//        VehicleType bicycle_slow = VehicleUtils.getFactory().createVehicleType(Id.create("bicycle_slow", VehicleType.class));
//        bicycle_slow.setMaximumVelocity(5);
//        bicycle_slow.setNetworkMode("bicycle");
//        sc.getVehicles().addVehicleType(bicycle_slow);
//
//        VehicleType bicycle_medium = VehicleUtils.getFactory().createVehicleType(Id.create("bicycle_medium", VehicleType.class));
//        bicycle_medium.setMaximumVelocity(11);
//        bicycle_medium.setNetworkMode("bicycle");
//        sc.getVehicles().addVehicleType(bicycle_medium);
//
//        VehicleType bicycle_fast = VehicleUtils.getFactory().createVehicleType(Id.create("bicycle_fast", VehicleType.class));
//        bicycle_fast.setMaximumVelocity(18);
//        bicycle_fast.setNetworkMode("bicycle");
//        sc.getVehicles().addVehicleType(bicycle_fast);


        Controler controler = new Controler(sc);
        controler.getConfig().qsim().setMainModes(Arrays.asList("car","bicycle_slow","bicycle_medium","bicycle_fast"));
        controler.getConfig().subtourModeChoice().setModes(new String[]{"car","bicycle_slow","bicycle_medium","bicycle_fast"});
        controler.getConfig().plansCalcRoute().setNetworkModes(Arrays.asList("car","bicycle_slow","bicycle_medium","bicycle_fast"));
        Set<String> analyzedModes = new HashSet<>();
        analyzedModes.add("car");
        analyzedModes.add("bicycle_slow");
        analyzedModes.add("bicycle_medium");
        analyzedModes.add("bicycle_fast");
        analyzedModes.add("walk");
        controler.getConfig().travelTimeCalculator().setAnalyzedModes(analyzedModes);

        for( Link link : sc.getNetwork().getLinks().values() ){
            if (link.getAllowedModes().contains("bicycle")) {
                // It would be better to set the modes directly in the input network file if not all car roads are open to rickshaw, cng, bus, hh etc.
                Set allowedModes = new HashSet(link.getAllowedModes());
                allowedModes.add("bicycle_slow");
                allowedModes.add("bicycle_medium");
                allowedModes.add("bicycle_fast");
                link.setAllowedModes( allowedModes );
            }
        }

        new VehicleWriterV1(sc.getVehicles()).writeFile(Paths.get("data","amsterdam","original_input","vehicles.xml").toString());
        return controler;
    }


}
